// Copyright (c) Microsoft Corporation. All rights reserved.
// Licensed under the MIT License. See License.txt in the project root for license information.
// 
// Code generated by Microsoft (R) AutoRest Code Generator.
// Changes may cause incorrect behavior and will be lost if the code is
// regenerated.

//
// This file was autogenerated by a tool.
// Do not modify it.
//

namespace Microsoft.Azure.Batch
{
    using Models = Microsoft.Azure.Batch.Protocol.Models;
    using System;
    using System.Collections.Generic;
    using System.Linq;

    /// <summary>
    /// Information used to connect to an Azure Storage Container using Blobfuse.
    /// </summary>
    public partial class AzureBlobFileSystemConfiguration : ITransportObjectProvider<Models.AzureBlobFileSystemConfiguration>, IPropertyMetadata
    {
        #region Constructors
        /// <summary>
        /// Initializes a new instance of the <see cref="AzureBlobFileSystemConfiguration"/> class.
        /// </summary>
        /// <param name='accountName'>The Azure Storage account name.</param>
        /// <param name='containerName'>The Azure Blob Storage Container name.</param>
        /// <param name='relativeMountPath'>The relative path on the compute node where the file system will be mounted.</param>
        /// <param name='accountKey'>The Azure Storage Account key. This property is mutually exclusive with <see cref="SasKey"/>.</param>
        /// <param name='sasKey'>The Azure Storage SAS token. This property is mutually exclusive with <see cref="AccountKey"/>.</param>
        /// <param name='blobfuseOptions'>Additional command line options to pass to the mount command.</param>
        internal AzureBlobFileSystemConfiguration(
            string accountName,
            string containerName,
            string relativeMountPath,
            string accountKey = default(string),
            string sasKey = default(string),
            string blobfuseOptions = default(string))
        {
            this.AccountName = accountName;
            this.ContainerName = containerName;
            this.RelativeMountPath = relativeMountPath;
            this.AccountKey = accountKey;
            this.SasKey = sasKey;
            this.BlobfuseOptions = blobfuseOptions;
        }

        internal AzureBlobFileSystemConfiguration(Models.AzureBlobFileSystemConfiguration protocolObject)
        {
            this.AccountKey = protocolObject.AccountKey;
            this.AccountName = protocolObject.AccountName;
            this.BlobfuseOptions = protocolObject.BlobfuseOptions;
            this.ContainerName = protocolObject.ContainerName;
            this.RelativeMountPath = protocolObject.RelativeMountPath;
            this.SasKey = protocolObject.SasKey;
        }

        #endregion Constructors

        #region AzureBlobFileSystemConfiguration

        /// <summary>
        /// Gets the Azure Storage Account key. This property is mutually exclusive with <see cref="SasKey"/>.
        /// </summary>
        public string AccountKey { get; }

        /// <summary>
        /// Gets the Azure Storage account name.
        /// </summary>
        public string AccountName { get; }

        /// <summary>
        /// Gets additional command line options to pass to the mount command.
        /// </summary>
        /// <remarks>
        /// These are 'net use' options in Windows and 'mount' options in Linux.
        /// </remarks>
        public string BlobfuseOptions { get; }

        /// <summary>
        /// Gets the Azure Blob Storage Container name.
        /// </summary>
        public string ContainerName { get; }

        /// <summary>
        /// Gets the relative path on the compute node where the file system will be mounted.
        /// </summary>
        /// <remarks>
        /// All file systems are mounted relative to the Batch mounts directory, accessible via the AZ_BATCH_NODE_MOUNTS_DIR 
        /// environment variable.
        /// </remarks>
        public string RelativeMountPath { get; }

        /// <summary>
        /// Gets the Azure Storage SAS token. This property is mutually exclusive with <see cref="AccountKey"/>.
        /// </summary>
        public string SasKey { get; }

        #endregion // AzureBlobFileSystemConfiguration

        #region IPropertyMetadata

        bool IModifiable.HasBeenModified
        {
            //This class is compile time readonly so it cannot have been modified
            get { return false; }
        }

        bool IReadOnly.IsReadOnly
        {
            get { return true; }
            set
            {
                // This class is compile time readonly already
            }
        }

        #endregion // IPropertyMetadata

        #region Internal/private methods

        /// <summary>
        /// Return a protocol object of the requested type.
        /// </summary>
        /// <returns>The protocol object of the requested type.</returns>
        Models.AzureBlobFileSystemConfiguration ITransportObjectProvider<Models.AzureBlobFileSystemConfiguration>.GetTransportObject()
        {
            Models.AzureBlobFileSystemConfiguration result = new Models.AzureBlobFileSystemConfiguration()
            {
                AccountKey = this.AccountKey,
                AccountName = this.AccountName,
                BlobfuseOptions = this.BlobfuseOptions,
                ContainerName = this.ContainerName,
                RelativeMountPath = this.RelativeMountPath,
                SasKey = this.SasKey,
            };

            return result;
        }


        #endregion // Internal/private methods
    }
}