// Copyright (c) Microsoft Corporation. All rights reserved.
// Licensed under the MIT License. See License.txt in the project root for
// license information.
//
// Code generated by Microsoft (R) AutoRest Code Generator 1.0.0.0
// Changes may cause incorrect behavior and will be lost if the code is
// regenerated.

namespace Microsoft.Azure.Management.CustomerInsights
{
    using Azure;
    using Management;
    using Rest;
    using Rest.Azure;
    using Models;
    using System.Threading;
    using System.Threading.Tasks;

    /// <summary>
    /// Extension methods for ConnectorMappingsOperations.
    /// </summary>
    public static partial class ConnectorMappingsOperationsExtensions
    {
            /// <summary>
            /// Creates a connector mapping or updates an existing connector mapping in the
            /// connector.
            /// </summary>
            /// <param name='operations'>
            /// The operations group for this extension method.
            /// </param>
            /// <param name='resourceGroupName'>
            /// The name of the resource group.
            /// </param>
            /// <param name='hubName'>
            /// The name of the hub.
            /// </param>
            /// <param name='connectorName'>
            /// The name of the connector.
            /// </param>
            /// <param name='mappingName'>
            /// The name of the connector mapping.
            /// </param>
            /// <param name='parameters'>
            /// Parameters supplied to the CreateOrUpdate Connector Mapping operation.
            /// </param>
            public static ConnectorMappingResourceFormat CreateOrUpdate(this IConnectorMappingsOperations operations, string resourceGroupName, string hubName, string connectorName, string mappingName, ConnectorMappingResourceFormat parameters)
            {
                return operations.CreateOrUpdateAsync(resourceGroupName, hubName, connectorName, mappingName, parameters).GetAwaiter().GetResult();
            }

            /// <summary>
            /// Creates a connector mapping or updates an existing connector mapping in the
            /// connector.
            /// </summary>
            /// <param name='operations'>
            /// The operations group for this extension method.
            /// </param>
            /// <param name='resourceGroupName'>
            /// The name of the resource group.
            /// </param>
            /// <param name='hubName'>
            /// The name of the hub.
            /// </param>
            /// <param name='connectorName'>
            /// The name of the connector.
            /// </param>
            /// <param name='mappingName'>
            /// The name of the connector mapping.
            /// </param>
            /// <param name='parameters'>
            /// Parameters supplied to the CreateOrUpdate Connector Mapping operation.
            /// </param>
            /// <param name='cancellationToken'>
            /// The cancellation token.
            /// </param>
            public static async Task<ConnectorMappingResourceFormat> CreateOrUpdateAsync(this IConnectorMappingsOperations operations, string resourceGroupName, string hubName, string connectorName, string mappingName, ConnectorMappingResourceFormat parameters, CancellationToken cancellationToken = default(CancellationToken))
            {
                using (var _result = await operations.CreateOrUpdateWithHttpMessagesAsync(resourceGroupName, hubName, connectorName, mappingName, parameters, null, cancellationToken).ConfigureAwait(false))
                {
                    return _result.Body;
                }
            }

            /// <summary>
            /// Gets a connector mapping in the connector.
            /// </summary>
            /// <param name='operations'>
            /// The operations group for this extension method.
            /// </param>
            /// <param name='resourceGroupName'>
            /// The name of the resource group.
            /// </param>
            /// <param name='hubName'>
            /// The name of the hub.
            /// </param>
            /// <param name='connectorName'>
            /// The name of the connector.
            /// </param>
            /// <param name='mappingName'>
            /// The name of the connector mapping.
            /// </param>
            public static ConnectorMappingResourceFormat Get(this IConnectorMappingsOperations operations, string resourceGroupName, string hubName, string connectorName, string mappingName)
            {
                return operations.GetAsync(resourceGroupName, hubName, connectorName, mappingName).GetAwaiter().GetResult();
            }

            /// <summary>
            /// Gets a connector mapping in the connector.
            /// </summary>
            /// <param name='operations'>
            /// The operations group for this extension method.
            /// </param>
            /// <param name='resourceGroupName'>
            /// The name of the resource group.
            /// </param>
            /// <param name='hubName'>
            /// The name of the hub.
            /// </param>
            /// <param name='connectorName'>
            /// The name of the connector.
            /// </param>
            /// <param name='mappingName'>
            /// The name of the connector mapping.
            /// </param>
            /// <param name='cancellationToken'>
            /// The cancellation token.
            /// </param>
            public static async Task<ConnectorMappingResourceFormat> GetAsync(this IConnectorMappingsOperations operations, string resourceGroupName, string hubName, string connectorName, string mappingName, CancellationToken cancellationToken = default(CancellationToken))
            {
                using (var _result = await operations.GetWithHttpMessagesAsync(resourceGroupName, hubName, connectorName, mappingName, null, cancellationToken).ConfigureAwait(false))
                {
                    return _result.Body;
                }
            }

            /// <summary>
            /// Deletes a connector mapping in the connector.
            /// </summary>
            /// <param name='operations'>
            /// The operations group for this extension method.
            /// </param>
            /// <param name='resourceGroupName'>
            /// The name of the resource group.
            /// </param>
            /// <param name='hubName'>
            /// The name of the hub.
            /// </param>
            /// <param name='connectorName'>
            /// The name of the connector.
            /// </param>
            /// <param name='mappingName'>
            /// The name of the connector mapping.
            /// </param>
            public static void Delete(this IConnectorMappingsOperations operations, string resourceGroupName, string hubName, string connectorName, string mappingName)
            {
                operations.DeleteAsync(resourceGroupName, hubName, connectorName, mappingName).GetAwaiter().GetResult();
            }

            /// <summary>
            /// Deletes a connector mapping in the connector.
            /// </summary>
            /// <param name='operations'>
            /// The operations group for this extension method.
            /// </param>
            /// <param name='resourceGroupName'>
            /// The name of the resource group.
            /// </param>
            /// <param name='hubName'>
            /// The name of the hub.
            /// </param>
            /// <param name='connectorName'>
            /// The name of the connector.
            /// </param>
            /// <param name='mappingName'>
            /// The name of the connector mapping.
            /// </param>
            /// <param name='cancellationToken'>
            /// The cancellation token.
            /// </param>
            public static async Task DeleteAsync(this IConnectorMappingsOperations operations, string resourceGroupName, string hubName, string connectorName, string mappingName, CancellationToken cancellationToken = default(CancellationToken))
            {
                await operations.DeleteWithHttpMessagesAsync(resourceGroupName, hubName, connectorName, mappingName, null, cancellationToken).ConfigureAwait(false);
            }

            /// <summary>
            /// Gets all the connector mappings in the specified connector.
            /// </summary>
            /// <param name='operations'>
            /// The operations group for this extension method.
            /// </param>
            /// <param name='resourceGroupName'>
            /// The name of the resource group.
            /// </param>
            /// <param name='hubName'>
            /// The name of the hub.
            /// </param>
            /// <param name='connectorName'>
            /// The name of the connector.
            /// </param>
            public static IPage<ConnectorMappingResourceFormat> ListByConnector(this IConnectorMappingsOperations operations, string resourceGroupName, string hubName, string connectorName)
            {
                return operations.ListByConnectorAsync(resourceGroupName, hubName, connectorName).GetAwaiter().GetResult();
            }

            /// <summary>
            /// Gets all the connector mappings in the specified connector.
            /// </summary>
            /// <param name='operations'>
            /// The operations group for this extension method.
            /// </param>
            /// <param name='resourceGroupName'>
            /// The name of the resource group.
            /// </param>
            /// <param name='hubName'>
            /// The name of the hub.
            /// </param>
            /// <param name='connectorName'>
            /// The name of the connector.
            /// </param>
            /// <param name='cancellationToken'>
            /// The cancellation token.
            /// </param>
            public static async Task<IPage<ConnectorMappingResourceFormat>> ListByConnectorAsync(this IConnectorMappingsOperations operations, string resourceGroupName, string hubName, string connectorName, CancellationToken cancellationToken = default(CancellationToken))
            {
                using (var _result = await operations.ListByConnectorWithHttpMessagesAsync(resourceGroupName, hubName, connectorName, null, cancellationToken).ConfigureAwait(false))
                {
                    return _result.Body;
                }
            }

            /// <summary>
            /// Gets all the connector mappings in the specified connector.
            /// </summary>
            /// <param name='operations'>
            /// The operations group for this extension method.
            /// </param>
            /// <param name='nextPageLink'>
            /// The NextLink from the previous successful call to List operation.
            /// </param>
            public static IPage<ConnectorMappingResourceFormat> ListByConnectorNext(this IConnectorMappingsOperations operations, string nextPageLink)
            {
                return operations.ListByConnectorNextAsync(nextPageLink).GetAwaiter().GetResult();
            }

            /// <summary>
            /// Gets all the connector mappings in the specified connector.
            /// </summary>
            /// <param name='operations'>
            /// The operations group for this extension method.
            /// </param>
            /// <param name='nextPageLink'>
            /// The NextLink from the previous successful call to List operation.
            /// </param>
            /// <param name='cancellationToken'>
            /// The cancellation token.
            /// </param>
            public static async Task<IPage<ConnectorMappingResourceFormat>> ListByConnectorNextAsync(this IConnectorMappingsOperations operations, string nextPageLink, CancellationToken cancellationToken = default(CancellationToken))
            {
                using (var _result = await operations.ListByConnectorNextWithHttpMessagesAsync(nextPageLink, null, cancellationToken).ConfigureAwait(false))
                {
                    return _result.Body;
                }
            }

    }
}

