// Copyright (c) Microsoft Corporation. All rights reserved.
// Licensed under the MIT License. See License.txt in the project root for
// license information.
//
// Code generated by Microsoft (R) AutoRest Code Generator 1.0.0.0
// Changes may cause incorrect behavior and will be lost if the code is
// regenerated.

namespace Microsoft.Azure.Management.CustomerInsights
{
    using Azure;
    using Management;
    using Rest;
    using Rest.Azure;
    using Models;
    using System.Threading;
    using System.Threading.Tasks;

    /// <summary>
    /// Extension methods for InteractionsOperations.
    /// </summary>
    public static partial class InteractionsOperationsExtensions
    {
            /// <summary>
            /// Creates an interaction or updates an existing interaction within a hub.
            /// </summary>
            /// <param name='operations'>
            /// The operations group for this extension method.
            /// </param>
            /// <param name='resourceGroupName'>
            /// The name of the resource group.
            /// </param>
            /// <param name='hubName'>
            /// The name of the hub.
            /// </param>
            /// <param name='interactionName'>
            /// The name of the interaction.
            /// </param>
            /// <param name='parameters'>
            /// Parameters supplied to the CreateOrUpdate Interaction operation.
            /// </param>
            public static InteractionResourceFormat CreateOrUpdate(this IInteractionsOperations operations, string resourceGroupName, string hubName, string interactionName, InteractionResourceFormat parameters)
            {
                return operations.CreateOrUpdateAsync(resourceGroupName, hubName, interactionName, parameters).GetAwaiter().GetResult();
            }

            /// <summary>
            /// Creates an interaction or updates an existing interaction within a hub.
            /// </summary>
            /// <param name='operations'>
            /// The operations group for this extension method.
            /// </param>
            /// <param name='resourceGroupName'>
            /// The name of the resource group.
            /// </param>
            /// <param name='hubName'>
            /// The name of the hub.
            /// </param>
            /// <param name='interactionName'>
            /// The name of the interaction.
            /// </param>
            /// <param name='parameters'>
            /// Parameters supplied to the CreateOrUpdate Interaction operation.
            /// </param>
            /// <param name='cancellationToken'>
            /// The cancellation token.
            /// </param>
            public static async Task<InteractionResourceFormat> CreateOrUpdateAsync(this IInteractionsOperations operations, string resourceGroupName, string hubName, string interactionName, InteractionResourceFormat parameters, CancellationToken cancellationToken = default(CancellationToken))
            {
                using (var _result = await operations.CreateOrUpdateWithHttpMessagesAsync(resourceGroupName, hubName, interactionName, parameters, null, cancellationToken).ConfigureAwait(false))
                {
                    return _result.Body;
                }
            }

            /// <summary>
            /// Gets information about the specified interaction.
            /// </summary>
            /// <param name='operations'>
            /// The operations group for this extension method.
            /// </param>
            /// <param name='resourceGroupName'>
            /// The name of the resource group.
            /// </param>
            /// <param name='hubName'>
            /// The name of the hub.
            /// </param>
            /// <param name='interactionName'>
            /// The name of the interaction.
            /// </param>
            /// <param name='localeCode'>
            /// Locale of interaction to retrieve, default is en-us.
            /// </param>
            public static InteractionResourceFormat Get(this IInteractionsOperations operations, string resourceGroupName, string hubName, string interactionName, string localeCode = "en-us")
            {
                return operations.GetAsync(resourceGroupName, hubName, interactionName, localeCode).GetAwaiter().GetResult();
            }

            /// <summary>
            /// Gets information about the specified interaction.
            /// </summary>
            /// <param name='operations'>
            /// The operations group for this extension method.
            /// </param>
            /// <param name='resourceGroupName'>
            /// The name of the resource group.
            /// </param>
            /// <param name='hubName'>
            /// The name of the hub.
            /// </param>
            /// <param name='interactionName'>
            /// The name of the interaction.
            /// </param>
            /// <param name='localeCode'>
            /// Locale of interaction to retrieve, default is en-us.
            /// </param>
            /// <param name='cancellationToken'>
            /// The cancellation token.
            /// </param>
            public static async Task<InteractionResourceFormat> GetAsync(this IInteractionsOperations operations, string resourceGroupName, string hubName, string interactionName, string localeCode = "en-us", CancellationToken cancellationToken = default(CancellationToken))
            {
                using (var _result = await operations.GetWithHttpMessagesAsync(resourceGroupName, hubName, interactionName, localeCode, null, cancellationToken).ConfigureAwait(false))
                {
                    return _result.Body;
                }
            }

            /// <summary>
            /// Gets all interactions in the hub.
            /// </summary>
            /// <param name='operations'>
            /// The operations group for this extension method.
            /// </param>
            /// <param name='resourceGroupName'>
            /// The name of the resource group.
            /// </param>
            /// <param name='hubName'>
            /// The name of the hub.
            /// </param>
            /// <param name='localeCode'>
            /// Locale of interaction to retrieve, default is en-us.
            /// </param>
            public static IPage<InteractionResourceFormat> ListByHub(this IInteractionsOperations operations, string resourceGroupName, string hubName, string localeCode = "en-us")
            {
                return operations.ListByHubAsync(resourceGroupName, hubName, localeCode).GetAwaiter().GetResult();
            }

            /// <summary>
            /// Gets all interactions in the hub.
            /// </summary>
            /// <param name='operations'>
            /// The operations group for this extension method.
            /// </param>
            /// <param name='resourceGroupName'>
            /// The name of the resource group.
            /// </param>
            /// <param name='hubName'>
            /// The name of the hub.
            /// </param>
            /// <param name='localeCode'>
            /// Locale of interaction to retrieve, default is en-us.
            /// </param>
            /// <param name='cancellationToken'>
            /// The cancellation token.
            /// </param>
            public static async Task<IPage<InteractionResourceFormat>> ListByHubAsync(this IInteractionsOperations operations, string resourceGroupName, string hubName, string localeCode = "en-us", CancellationToken cancellationToken = default(CancellationToken))
            {
                using (var _result = await operations.ListByHubWithHttpMessagesAsync(resourceGroupName, hubName, localeCode, null, cancellationToken).ConfigureAwait(false))
                {
                    return _result.Body;
                }
            }

            /// <summary>
            /// Suggests relationships to create relationship links.
            /// </summary>
            /// <param name='operations'>
            /// The operations group for this extension method.
            /// </param>
            /// <param name='resourceGroupName'>
            /// The name of the resource group.
            /// </param>
            /// <param name='hubName'>
            /// The name of the hub.
            /// </param>
            /// <param name='interactionName'>
            /// The name of the interaction.
            /// </param>
            public static SuggestRelationshipLinksResponse SuggestRelationshipLinks(this IInteractionsOperations operations, string resourceGroupName, string hubName, string interactionName)
            {
                return operations.SuggestRelationshipLinksAsync(resourceGroupName, hubName, interactionName).GetAwaiter().GetResult();
            }

            /// <summary>
            /// Suggests relationships to create relationship links.
            /// </summary>
            /// <param name='operations'>
            /// The operations group for this extension method.
            /// </param>
            /// <param name='resourceGroupName'>
            /// The name of the resource group.
            /// </param>
            /// <param name='hubName'>
            /// The name of the hub.
            /// </param>
            /// <param name='interactionName'>
            /// The name of the interaction.
            /// </param>
            /// <param name='cancellationToken'>
            /// The cancellation token.
            /// </param>
            public static async Task<SuggestRelationshipLinksResponse> SuggestRelationshipLinksAsync(this IInteractionsOperations operations, string resourceGroupName, string hubName, string interactionName, CancellationToken cancellationToken = default(CancellationToken))
            {
                using (var _result = await operations.SuggestRelationshipLinksWithHttpMessagesAsync(resourceGroupName, hubName, interactionName, null, cancellationToken).ConfigureAwait(false))
                {
                    return _result.Body;
                }
            }

            /// <summary>
            /// Creates an interaction or updates an existing interaction within a hub.
            /// </summary>
            /// <param name='operations'>
            /// The operations group for this extension method.
            /// </param>
            /// <param name='resourceGroupName'>
            /// The name of the resource group.
            /// </param>
            /// <param name='hubName'>
            /// The name of the hub.
            /// </param>
            /// <param name='interactionName'>
            /// The name of the interaction.
            /// </param>
            /// <param name='parameters'>
            /// Parameters supplied to the CreateOrUpdate Interaction operation.
            /// </param>
            public static InteractionResourceFormat BeginCreateOrUpdate(this IInteractionsOperations operations, string resourceGroupName, string hubName, string interactionName, InteractionResourceFormat parameters)
            {
                return operations.BeginCreateOrUpdateAsync(resourceGroupName, hubName, interactionName, parameters).GetAwaiter().GetResult();
            }

            /// <summary>
            /// Creates an interaction or updates an existing interaction within a hub.
            /// </summary>
            /// <param name='operations'>
            /// The operations group for this extension method.
            /// </param>
            /// <param name='resourceGroupName'>
            /// The name of the resource group.
            /// </param>
            /// <param name='hubName'>
            /// The name of the hub.
            /// </param>
            /// <param name='interactionName'>
            /// The name of the interaction.
            /// </param>
            /// <param name='parameters'>
            /// Parameters supplied to the CreateOrUpdate Interaction operation.
            /// </param>
            /// <param name='cancellationToken'>
            /// The cancellation token.
            /// </param>
            public static async Task<InteractionResourceFormat> BeginCreateOrUpdateAsync(this IInteractionsOperations operations, string resourceGroupName, string hubName, string interactionName, InteractionResourceFormat parameters, CancellationToken cancellationToken = default(CancellationToken))
            {
                using (var _result = await operations.BeginCreateOrUpdateWithHttpMessagesAsync(resourceGroupName, hubName, interactionName, parameters, null, cancellationToken).ConfigureAwait(false))
                {
                    return _result.Body;
                }
            }

            /// <summary>
            /// Gets all interactions in the hub.
            /// </summary>
            /// <param name='operations'>
            /// The operations group for this extension method.
            /// </param>
            /// <param name='nextPageLink'>
            /// The NextLink from the previous successful call to List operation.
            /// </param>
            public static IPage<InteractionResourceFormat> ListByHubNext(this IInteractionsOperations operations, string nextPageLink)
            {
                return operations.ListByHubNextAsync(nextPageLink).GetAwaiter().GetResult();
            }

            /// <summary>
            /// Gets all interactions in the hub.
            /// </summary>
            /// <param name='operations'>
            /// The operations group for this extension method.
            /// </param>
            /// <param name='nextPageLink'>
            /// The NextLink from the previous successful call to List operation.
            /// </param>
            /// <param name='cancellationToken'>
            /// The cancellation token.
            /// </param>
            public static async Task<IPage<InteractionResourceFormat>> ListByHubNextAsync(this IInteractionsOperations operations, string nextPageLink, CancellationToken cancellationToken = default(CancellationToken))
            {
                using (var _result = await operations.ListByHubNextWithHttpMessagesAsync(nextPageLink, null, cancellationToken).ConfigureAwait(false))
                {
                    return _result.Body;
                }
            }

    }
}

