// Copyright (c) Microsoft Corporation. All rights reserved.
// Licensed under the MIT License. See License.txt in the project root for
// license information.
//
// Code generated by Microsoft (R) AutoRest Code Generator 1.0.0.0
// Changes may cause incorrect behavior and will be lost if the code is
// regenerated.

namespace Microsoft.Azure.Management.CustomerInsights.Models
{
    using Azure;
    using Management;
    using CustomerInsights;
    using Rest;
    using Rest.Serialization;
    using Newtonsoft.Json;
    using System.Collections;
    using System.Collections.Generic;
    using System.Linq;

    /// <summary>
    /// The relationship link resource format.
    /// </summary>
    [JsonTransformation]
    public partial class RelationshipLinkResourceFormat : ProxyResource
    {
        /// <summary>
        /// Initializes a new instance of the RelationshipLinkResourceFormat
        /// class.
        /// </summary>
        public RelationshipLinkResourceFormat() { }

        /// <summary>
        /// Initializes a new instance of the RelationshipLinkResourceFormat
        /// class.
        /// </summary>
        /// <param name="interactionType">The InteractionType associated with
        /// the Relationship Link.</param>
        /// <param name="profilePropertyReferences">The property references for
        /// the Profile of the Relationship.</param>
        /// <param name="relatedProfilePropertyReferences">The property
        /// references for the Related Profile of the Relationship.</param>
        /// <param name="relationshipName">The Relationship associated with the
        /// Link.</param>
        /// <param name="id">Resource ID.</param>
        /// <param name="name">Resource name.</param>
        /// <param name="type">Resource type.</param>
        /// <param name="displayName">Localized display name for the
        /// Relationship Link.</param>
        /// <param name="description">Localized descriptions for the
        /// Relationship Link.</param>
        /// <param name="linkName">The name of the Relationship Link.</param>
        /// <param name="mappings">The mappings between Interaction and
        /// Relationship fields.</param>
        /// <param name="provisioningState">Provisioning state. Possible values
        /// include: 'Provisioning', 'Succeeded', 'Expiring', 'Deleting',
        /// 'HumanIntervention', 'Failed'</param>
        /// <param name="relationshipGuidId">The relationship guid id.</param>
        /// <param name="tenantId">The hub name.</param>
        public RelationshipLinkResourceFormat(string interactionType, IList<ParticipantPropertyReference> profilePropertyReferences, IList<ParticipantPropertyReference> relatedProfilePropertyReferences, string relationshipName, string id = default(string), string name = default(string), string type = default(string), IDictionary<string, string> displayName = default(IDictionary<string, string>), IDictionary<string, string> description = default(IDictionary<string, string>), string linkName = default(string), IList<RelationshipLinkFieldMapping> mappings = default(IList<RelationshipLinkFieldMapping>), string provisioningState = default(string), string relationshipGuidId = default(string), string tenantId = default(string))
            : base(id, name, type)
        {
            DisplayName = displayName;
            Description = description;
            InteractionType = interactionType;
            LinkName = linkName;
            Mappings = mappings;
            ProfilePropertyReferences = profilePropertyReferences;
            ProvisioningState = provisioningState;
            RelatedProfilePropertyReferences = relatedProfilePropertyReferences;
            RelationshipName = relationshipName;
            RelationshipGuidId = relationshipGuidId;
            TenantId = tenantId;
        }

        /// <summary>
        /// Gets or sets localized display name for the Relationship Link.
        /// </summary>
        [JsonProperty(PropertyName = "properties.displayName")]
        public IDictionary<string, string> DisplayName { get; set; }

        /// <summary>
        /// Gets or sets localized descriptions for the Relationship Link.
        /// </summary>
        [JsonProperty(PropertyName = "properties.description")]
        public IDictionary<string, string> Description { get; set; }

        /// <summary>
        /// Gets or sets the InteractionType associated with the Relationship
        /// Link.
        /// </summary>
        [JsonProperty(PropertyName = "properties.interactionType")]
        public string InteractionType { get; set; }

        /// <summary>
        /// Gets the name of the Relationship Link.
        /// </summary>
        [JsonProperty(PropertyName = "properties.linkName")]
        public string LinkName { get; protected set; }

        /// <summary>
        /// Gets or sets the mappings between Interaction and Relationship
        /// fields.
        /// </summary>
        [JsonProperty(PropertyName = "properties.mappings")]
        public IList<RelationshipLinkFieldMapping> Mappings { get; set; }

        /// <summary>
        /// Gets or sets the property references for the Profile of the
        /// Relationship.
        /// </summary>
        [JsonProperty(PropertyName = "properties.profilePropertyReferences")]
        public IList<ParticipantPropertyReference> ProfilePropertyReferences { get; set; }

        /// <summary>
        /// Gets provisioning state. Possible values include: 'Provisioning',
        /// 'Succeeded', 'Expiring', 'Deleting', 'HumanIntervention', 'Failed'
        /// </summary>
        [JsonProperty(PropertyName = "properties.provisioningState")]
        public string ProvisioningState { get; protected set; }

        /// <summary>
        /// Gets or sets the property references for the Related Profile of the
        /// Relationship.
        /// </summary>
        [JsonProperty(PropertyName = "properties.relatedProfilePropertyReferences")]
        public IList<ParticipantPropertyReference> RelatedProfilePropertyReferences { get; set; }

        /// <summary>
        /// Gets or sets the Relationship associated with the Link.
        /// </summary>
        [JsonProperty(PropertyName = "properties.relationshipName")]
        public string RelationshipName { get; set; }

        /// <summary>
        /// Gets the relationship guid id.
        /// </summary>
        [JsonProperty(PropertyName = "properties.relationshipGuidId")]
        public string RelationshipGuidId { get; protected set; }

        /// <summary>
        /// Gets the hub name.
        /// </summary>
        [JsonProperty(PropertyName = "properties.tenantId")]
        public string TenantId { get; protected set; }

        /// <summary>
        /// Validate the object.
        /// </summary>
        /// <exception cref="ValidationException">
        /// Thrown if validation fails
        /// </exception>
        public virtual void Validate()
        {
            if (InteractionType == null)
            {
                throw new ValidationException(ValidationRules.CannotBeNull, "InteractionType");
            }
            if (ProfilePropertyReferences == null)
            {
                throw new ValidationException(ValidationRules.CannotBeNull, "ProfilePropertyReferences");
            }
            if (RelatedProfilePropertyReferences == null)
            {
                throw new ValidationException(ValidationRules.CannotBeNull, "RelatedProfilePropertyReferences");
            }
            if (RelationshipName == null)
            {
                throw new ValidationException(ValidationRules.CannotBeNull, "RelationshipName");
            }
            if (Mappings != null)
            {
                foreach (var element in Mappings)
                {
                    if (element != null)
                    {
                        element.Validate();
                    }
                }
            }
            if (ProfilePropertyReferences != null)
            {
                foreach (var element1 in ProfilePropertyReferences)
                {
                    if (element1 != null)
                    {
                        element1.Validate();
                    }
                }
            }
            if (RelatedProfilePropertyReferences != null)
            {
                foreach (var element2 in RelatedProfilePropertyReferences)
                {
                    if (element2 != null)
                    {
                        element2.Validate();
                    }
                }
            }
        }
    }
}

