// Copyright (c) Microsoft Corporation. All rights reserved.
// Licensed under the MIT License. See License.txt in the project root for
// license information.
//
// Code generated by Microsoft (R) AutoRest Code Generator 1.0.0.0
// Changes may cause incorrect behavior and will be lost if the code is
// regenerated.

namespace Microsoft.Azure.Management.CustomerInsights
{
    using Azure;
    using Management;
    using Rest;
    using Rest.Azure;
    using Models;
    using System.Threading;
    using System.Threading.Tasks;

    /// <summary>
    /// Extension methods for RelationshipLinksOperations.
    /// </summary>
    public static partial class RelationshipLinksOperationsExtensions
    {
            /// <summary>
            /// Creates a relationship link or updates an existing relationship link within
            /// a hub.
            /// </summary>
            /// <param name='operations'>
            /// The operations group for this extension method.
            /// </param>
            /// <param name='resourceGroupName'>
            /// The name of the resource group.
            /// </param>
            /// <param name='hubName'>
            /// The name of the hub.
            /// </param>
            /// <param name='relationshipLinkName'>
            /// The name of the relationship link.
            /// </param>
            /// <param name='parameters'>
            /// Parameters supplied to the CreateOrUpdate relationship link operation.
            /// </param>
            public static RelationshipLinkResourceFormat CreateOrUpdate(this IRelationshipLinksOperations operations, string resourceGroupName, string hubName, string relationshipLinkName, RelationshipLinkResourceFormat parameters)
            {
                return operations.CreateOrUpdateAsync(resourceGroupName, hubName, relationshipLinkName, parameters).GetAwaiter().GetResult();
            }

            /// <summary>
            /// Creates a relationship link or updates an existing relationship link within
            /// a hub.
            /// </summary>
            /// <param name='operations'>
            /// The operations group for this extension method.
            /// </param>
            /// <param name='resourceGroupName'>
            /// The name of the resource group.
            /// </param>
            /// <param name='hubName'>
            /// The name of the hub.
            /// </param>
            /// <param name='relationshipLinkName'>
            /// The name of the relationship link.
            /// </param>
            /// <param name='parameters'>
            /// Parameters supplied to the CreateOrUpdate relationship link operation.
            /// </param>
            /// <param name='cancellationToken'>
            /// The cancellation token.
            /// </param>
            public static async Task<RelationshipLinkResourceFormat> CreateOrUpdateAsync(this IRelationshipLinksOperations operations, string resourceGroupName, string hubName, string relationshipLinkName, RelationshipLinkResourceFormat parameters, CancellationToken cancellationToken = default(CancellationToken))
            {
                using (var _result = await operations.CreateOrUpdateWithHttpMessagesAsync(resourceGroupName, hubName, relationshipLinkName, parameters, null, cancellationToken).ConfigureAwait(false))
                {
                    return _result.Body;
                }
            }

            /// <summary>
            /// Gets information about the specified relationship Link.
            /// </summary>
            /// <param name='operations'>
            /// The operations group for this extension method.
            /// </param>
            /// <param name='resourceGroupName'>
            /// The name of the resource group.
            /// </param>
            /// <param name='hubName'>
            /// The name of the hub.
            /// </param>
            /// <param name='relationshipLinkName'>
            /// The name of the relationship link.
            /// </param>
            public static RelationshipLinkResourceFormat Get(this IRelationshipLinksOperations operations, string resourceGroupName, string hubName, string relationshipLinkName)
            {
                return operations.GetAsync(resourceGroupName, hubName, relationshipLinkName).GetAwaiter().GetResult();
            }

            /// <summary>
            /// Gets information about the specified relationship Link.
            /// </summary>
            /// <param name='operations'>
            /// The operations group for this extension method.
            /// </param>
            /// <param name='resourceGroupName'>
            /// The name of the resource group.
            /// </param>
            /// <param name='hubName'>
            /// The name of the hub.
            /// </param>
            /// <param name='relationshipLinkName'>
            /// The name of the relationship link.
            /// </param>
            /// <param name='cancellationToken'>
            /// The cancellation token.
            /// </param>
            public static async Task<RelationshipLinkResourceFormat> GetAsync(this IRelationshipLinksOperations operations, string resourceGroupName, string hubName, string relationshipLinkName, CancellationToken cancellationToken = default(CancellationToken))
            {
                using (var _result = await operations.GetWithHttpMessagesAsync(resourceGroupName, hubName, relationshipLinkName, null, cancellationToken).ConfigureAwait(false))
                {
                    return _result.Body;
                }
            }

            /// <summary>
            /// Deletes a relationship link within a hub.
            /// </summary>
            /// <param name='operations'>
            /// The operations group for this extension method.
            /// </param>
            /// <param name='resourceGroupName'>
            /// The name of the resource group.
            /// </param>
            /// <param name='hubName'>
            /// The name of the hub.
            /// </param>
            /// <param name='relationshipLinkName'>
            /// The name of the relationship.
            /// </param>
            public static void Delete(this IRelationshipLinksOperations operations, string resourceGroupName, string hubName, string relationshipLinkName)
            {
                operations.DeleteAsync(resourceGroupName, hubName, relationshipLinkName).GetAwaiter().GetResult();
            }

            /// <summary>
            /// Deletes a relationship link within a hub.
            /// </summary>
            /// <param name='operations'>
            /// The operations group for this extension method.
            /// </param>
            /// <param name='resourceGroupName'>
            /// The name of the resource group.
            /// </param>
            /// <param name='hubName'>
            /// The name of the hub.
            /// </param>
            /// <param name='relationshipLinkName'>
            /// The name of the relationship.
            /// </param>
            /// <param name='cancellationToken'>
            /// The cancellation token.
            /// </param>
            public static async Task DeleteAsync(this IRelationshipLinksOperations operations, string resourceGroupName, string hubName, string relationshipLinkName, CancellationToken cancellationToken = default(CancellationToken))
            {
                await operations.DeleteWithHttpMessagesAsync(resourceGroupName, hubName, relationshipLinkName, null, cancellationToken).ConfigureAwait(false);
            }

            /// <summary>
            /// Gets all relationship links in the hub.
            /// </summary>
            /// <param name='operations'>
            /// The operations group for this extension method.
            /// </param>
            /// <param name='resourceGroupName'>
            /// The name of the resource group.
            /// </param>
            /// <param name='hubName'>
            /// The name of the hub.
            /// </param>
            public static IPage<RelationshipLinkResourceFormat> ListByHub(this IRelationshipLinksOperations operations, string resourceGroupName, string hubName)
            {
                return operations.ListByHubAsync(resourceGroupName, hubName).GetAwaiter().GetResult();
            }

            /// <summary>
            /// Gets all relationship links in the hub.
            /// </summary>
            /// <param name='operations'>
            /// The operations group for this extension method.
            /// </param>
            /// <param name='resourceGroupName'>
            /// The name of the resource group.
            /// </param>
            /// <param name='hubName'>
            /// The name of the hub.
            /// </param>
            /// <param name='cancellationToken'>
            /// The cancellation token.
            /// </param>
            public static async Task<IPage<RelationshipLinkResourceFormat>> ListByHubAsync(this IRelationshipLinksOperations operations, string resourceGroupName, string hubName, CancellationToken cancellationToken = default(CancellationToken))
            {
                using (var _result = await operations.ListByHubWithHttpMessagesAsync(resourceGroupName, hubName, null, cancellationToken).ConfigureAwait(false))
                {
                    return _result.Body;
                }
            }

            /// <summary>
            /// Creates a relationship link or updates an existing relationship link within
            /// a hub.
            /// </summary>
            /// <param name='operations'>
            /// The operations group for this extension method.
            /// </param>
            /// <param name='resourceGroupName'>
            /// The name of the resource group.
            /// </param>
            /// <param name='hubName'>
            /// The name of the hub.
            /// </param>
            /// <param name='relationshipLinkName'>
            /// The name of the relationship link.
            /// </param>
            /// <param name='parameters'>
            /// Parameters supplied to the CreateOrUpdate relationship link operation.
            /// </param>
            public static RelationshipLinkResourceFormat BeginCreateOrUpdate(this IRelationshipLinksOperations operations, string resourceGroupName, string hubName, string relationshipLinkName, RelationshipLinkResourceFormat parameters)
            {
                return operations.BeginCreateOrUpdateAsync(resourceGroupName, hubName, relationshipLinkName, parameters).GetAwaiter().GetResult();
            }

            /// <summary>
            /// Creates a relationship link or updates an existing relationship link within
            /// a hub.
            /// </summary>
            /// <param name='operations'>
            /// The operations group for this extension method.
            /// </param>
            /// <param name='resourceGroupName'>
            /// The name of the resource group.
            /// </param>
            /// <param name='hubName'>
            /// The name of the hub.
            /// </param>
            /// <param name='relationshipLinkName'>
            /// The name of the relationship link.
            /// </param>
            /// <param name='parameters'>
            /// Parameters supplied to the CreateOrUpdate relationship link operation.
            /// </param>
            /// <param name='cancellationToken'>
            /// The cancellation token.
            /// </param>
            public static async Task<RelationshipLinkResourceFormat> BeginCreateOrUpdateAsync(this IRelationshipLinksOperations operations, string resourceGroupName, string hubName, string relationshipLinkName, RelationshipLinkResourceFormat parameters, CancellationToken cancellationToken = default(CancellationToken))
            {
                using (var _result = await operations.BeginCreateOrUpdateWithHttpMessagesAsync(resourceGroupName, hubName, relationshipLinkName, parameters, null, cancellationToken).ConfigureAwait(false))
                {
                    return _result.Body;
                }
            }

            /// <summary>
            /// Deletes a relationship link within a hub.
            /// </summary>
            /// <param name='operations'>
            /// The operations group for this extension method.
            /// </param>
            /// <param name='resourceGroupName'>
            /// The name of the resource group.
            /// </param>
            /// <param name='hubName'>
            /// The name of the hub.
            /// </param>
            /// <param name='relationshipLinkName'>
            /// The name of the relationship.
            /// </param>
            public static void BeginDelete(this IRelationshipLinksOperations operations, string resourceGroupName, string hubName, string relationshipLinkName)
            {
                operations.BeginDeleteAsync(resourceGroupName, hubName, relationshipLinkName).GetAwaiter().GetResult();
            }

            /// <summary>
            /// Deletes a relationship link within a hub.
            /// </summary>
            /// <param name='operations'>
            /// The operations group for this extension method.
            /// </param>
            /// <param name='resourceGroupName'>
            /// The name of the resource group.
            /// </param>
            /// <param name='hubName'>
            /// The name of the hub.
            /// </param>
            /// <param name='relationshipLinkName'>
            /// The name of the relationship.
            /// </param>
            /// <param name='cancellationToken'>
            /// The cancellation token.
            /// </param>
            public static async Task BeginDeleteAsync(this IRelationshipLinksOperations operations, string resourceGroupName, string hubName, string relationshipLinkName, CancellationToken cancellationToken = default(CancellationToken))
            {
                await operations.BeginDeleteWithHttpMessagesAsync(resourceGroupName, hubName, relationshipLinkName, null, cancellationToken).ConfigureAwait(false);
            }

            /// <summary>
            /// Gets all relationship links in the hub.
            /// </summary>
            /// <param name='operations'>
            /// The operations group for this extension method.
            /// </param>
            /// <param name='nextPageLink'>
            /// The NextLink from the previous successful call to List operation.
            /// </param>
            public static IPage<RelationshipLinkResourceFormat> ListByHubNext(this IRelationshipLinksOperations operations, string nextPageLink)
            {
                return operations.ListByHubNextAsync(nextPageLink).GetAwaiter().GetResult();
            }

            /// <summary>
            /// Gets all relationship links in the hub.
            /// </summary>
            /// <param name='operations'>
            /// The operations group for this extension method.
            /// </param>
            /// <param name='nextPageLink'>
            /// The NextLink from the previous successful call to List operation.
            /// </param>
            /// <param name='cancellationToken'>
            /// The cancellation token.
            /// </param>
            public static async Task<IPage<RelationshipLinkResourceFormat>> ListByHubNextAsync(this IRelationshipLinksOperations operations, string nextPageLink, CancellationToken cancellationToken = default(CancellationToken))
            {
                using (var _result = await operations.ListByHubNextWithHttpMessagesAsync(nextPageLink, null, cancellationToken).ConfigureAwait(false))
                {
                    return _result.Body;
                }
            }

    }
}

