---
page_type: sample
languages:
- csharp
products:
- azure
- azure-event-hubs
name: Azure.Messaging.EventHubs.Processor samples for .NET
description: Samples for the Azure.Messaging.EventHubs.Processor client library
---

# Azure.Messaging.EventHubs.Processor Samples

The  Azure Event Hubs Processor samples are intended to serve as an example and introduction to common scenarios in which the Event Hubs Processor client library is used, and to help demonstrate library features.  The samples are accompanied by a [console application](./Program.cs) which you can use to execute and debug them interactively.  The simplest way to begin is to launch the project for debugging in Visual Studio or your preferred IDE and provide the Event Hubs connection information in response to the prompts.

Each of the samples is self-contained and focused on illustrating one specific scenario.  Each is numbered, with the lower numbers concentrating on basic scenarios and building to more complex scenarios as they increase; though each sample is independent, it will assume an understanding of the content discussed in earlier samples.

## Getting started

- **Microsoft Azure Subscription:**  To use Azure services, including Azure Event Hubs, you'll need a subscription.  If you do not have an existing Azure account, you may sign up for a free trial or use your MSDN subscriber benefits when you [create an account](https://account.windowsazure.com/Home/Index).

- **Event Hubs namespace with an Event Hub:** To interact with Azure Event Hubs, you'll also need to have a namespace and Event Hub available.  If you are not familiar with creating Azure resources, you may wish to follow the step-by-step guide for [creating an Event Hub using the Azure portal](https://docs.microsoft.com/en-us/azure/event-hubs/event-hubs-create).  There, you can also find detailed instructions for using the Azure CLI, Azure PowerShell, or Azure Resource Manager (ARM) templates to create an Event Hub.

- **Azure Storage account with blob storage:** To persist checkpoints as blobs in Azure Storage, you'll need to have an Azure Storage account with blobs available.  If you are not familiar with Azure Storage accounts, you may wish to follow the step-by-step guide for [creating a storage account using the Azure portal](https://docs.microsoft.com/en-us/azure/storage/common/storage-quickstart-create-account?toc=%2Fazure%2Fstorage%2Fblobs%2Ftoc.json&tabs=azure-portal).  There, you can also find detailed instructions for using the Azure CLI, Azure PowerShell, or Azure Resource Manager (ARM) templates to create storage accounts.

- **C# 8.0:** The Azure Event Hubs client library makes use of new features that were introduced in C# 8.0.  You can still use the library with older versions of C#, but some of its functionality won't be available.  In order to enable these features, you need to [target .NET Core 3.0](https://docs.microsoft.com/en-us/dotnet/standard/frameworks#how-to-specify-target-frameworks) or [specify the language version](https://docs.microsoft.com/en-gb/dotnet/csharp/language-reference/configure-language-version#override-a-default) you want to use (8.0 or above).  If you are using Visual Studio, versions prior to Visual Studio 2019 are not compatible with the tools needed to build C# 8.0 projects.  Visual Studio 2019, including the free Community edition, can be downloaded [here](https://visualstudio.microsoft.com/vs/).

  **Important Note:** The use of C# 8.0 is mandatory to run the samples without modification.  You can still run the samples if you decide to tweak them.

To quickly create the needed resources in Azure and to receive connection strings for them, you can deploy our sample template by clicking:  

[![](http://azuredeploy.net/deploybutton.png)](https://portal.azure.com/#create/Microsoft.Template/uri/https%3A%2F%2Fraw.githubusercontent.com%2FAzure%2Fazure-sdk-for-net%2Fmaster%2Fsdk%2Feventhub%2FAzure.Messaging.EventHubs.Processor%2Fassets%2Fsamples-azure-deploy.json)

## Available samples

- [Hello world](./Sample01_HelloWorld.cs)  
  An introduction to the Event Processor client, illustrating how to create the client and perform basic operations.
  
- [Create an Event Processor client with custom options](./Sample02_ProcessorWithCustomOptions.cs)  
  An introduction to the Event Processor client, exploring additional options for creating the processor.

- [Perform basic event processing](./Sample03_BasicEventProcessing.cs)  
  An introduction to the Event Processor client, illustrating how to perform basic event processing.

- [Create checkpoints to track processing state](./Sample04_BasicCheckpointing.cs)  
  An introduction to the Event Processor client, illustrating how to create simple checkpoints.
  
- [Initialize an Event Hub partition for processing by a specific Event Processor client](./Sample05_InitializeAPartition.cs)  
  An introduction to the Event Processor client, illustrating how to participate in initialization for a partition.

- [Track when an Event Hub partition will no longer be processed by a specific Event Processor client](./Sample06_TrackWhenAPartitionIsClosed.cs)  
  An introduction to the Event Processor client, illustrating how to track when processing stops for a partition.
  
- [Manage the Event Processor when an error is encountered](./Sample07_RestartProcessingOnError.cs)  
  An example of stopping and restarting the Event Processor client when a specific error is encountered.

- [Send a heartbeat for health monitoring while processing events](./Sample08_EventProcessingHeartbeat.cs)  
  An example of ensuring that the handler for processing events is invoked on a fixed interval when no events are available.

- [Process events in batches](./Sample09_ProcessEventsByBatch.cs)  
  An example of grouping events into batches for downstream processing.
  
## Contributing  

This project welcomes contributions and suggestions.  Most contributions require you to agree to a Contributor License Agreement (CLA) declaring that you have the right to, and actually do, grant us the rights to use your contribution. For details, visit https://cla.microsoft.com.

When you submit a pull request, a CLA-bot will automatically determine whether you need to provide a CLA and decorate the PR appropriately (e.g., label, comment). Simply follow the instructions provided by the bot. You will only need to do this once across all repos using our CLA.

This project has adopted the [Microsoft Open Source Code of Conduct](https://opensource.microsoft.com/codeofconduct/). For more information see the [Code of Conduct FAQ](https://opensource.microsoft.com/codeofconduct/faq/) or contact [opencode@microsoft.com](mailto:opencode@microsoft.com) with any additional questions or comments.

Please see our [contributing guide](./../Azure.Messaging.EventHubs/CONTRIBUTING.md) for more information.
  
![Impressions](https://azure-sdk-impressions.azurewebsites.net/api/impressions/azure-sdk-for-net%2Fsdk%2Feventhub%2FAzure.Messaging.EventHubs.Processor/samples/%2FREADME.png)