// <auto-generated>
// Copyright (c) Microsoft Corporation. All rights reserved.
// Licensed under the MIT License. See License.txt in the project root for
// license information.
//
// Code generated by Microsoft (R) AutoRest Code Generator.
// Changes may cause incorrect behavior and will be lost if the code is
// regenerated.
// </auto-generated>

namespace Microsoft.Azure.Management.FrontDoor.Models
{
    using Microsoft.Rest;
    using Newtonsoft.Json;
    using System.Collections;
    using System.Collections.Generic;
    using System.Linq;

    /// <summary>
    /// Defines contents of a web application rule
    /// </summary>
    public partial class CustomRule
    {
        /// <summary>
        /// Initializes a new instance of the CustomRule class.
        /// </summary>
        public CustomRule()
        {
            CustomInit();
        }

        /// <summary>
        /// Initializes a new instance of the CustomRule class.
        /// </summary>
        /// <param name="priority">Describes priority of the rule. Rules with a
        /// lower value will be evaluated before rules with a higher
        /// value.</param>
        /// <param name="ruleType">Describes type of rule. Possible values
        /// include: 'MatchRule', 'RateLimitRule'</param>
        /// <param name="matchConditions">List of match conditions.</param>
        /// <param name="action">Describes what action to be applied when rule
        /// matches. Possible values include: 'Allow', 'Block', 'Log',
        /// 'Redirect'</param>
        /// <param name="name">Describes the name of the rule.</param>
        /// <param name="enabledState">Describes if the custom rule is in
        /// enabled or disabled state. Defaults to Enabled if not specified.
        /// Possible values include: 'Disabled', 'Enabled'</param>
        /// <param name="rateLimitDurationInMinutes">Time window for resetting
        /// the rate limit count. Default is 1 minute.</param>
        /// <param name="rateLimitThreshold">Number of allowed requests per
        /// client within the time window.</param>
        public CustomRule(int priority, string ruleType, IList<MatchCondition> matchConditions, string action, string name = default(string), string enabledState = default(string), int? rateLimitDurationInMinutes = default(int?), int? rateLimitThreshold = default(int?))
        {
            Name = name;
            Priority = priority;
            EnabledState = enabledState;
            RuleType = ruleType;
            RateLimitDurationInMinutes = rateLimitDurationInMinutes;
            RateLimitThreshold = rateLimitThreshold;
            MatchConditions = matchConditions;
            Action = action;
            CustomInit();
        }

        /// <summary>
        /// An initialization method that performs custom operations like setting defaults
        /// </summary>
        partial void CustomInit();

        /// <summary>
        /// Gets or sets describes the name of the rule.
        /// </summary>
        [JsonProperty(PropertyName = "name")]
        public string Name { get; set; }

        /// <summary>
        /// Gets or sets describes priority of the rule. Rules with a lower
        /// value will be evaluated before rules with a higher value.
        /// </summary>
        [JsonProperty(PropertyName = "priority")]
        public int Priority { get; set; }

        /// <summary>
        /// Gets or sets describes if the custom rule is in enabled or disabled
        /// state. Defaults to Enabled if not specified. Possible values
        /// include: 'Disabled', 'Enabled'
        /// </summary>
        [JsonProperty(PropertyName = "enabledState")]
        public string EnabledState { get; set; }

        /// <summary>
        /// Gets or sets describes type of rule. Possible values include:
        /// 'MatchRule', 'RateLimitRule'
        /// </summary>
        [JsonProperty(PropertyName = "ruleType")]
        public string RuleType { get; set; }

        /// <summary>
        /// Gets or sets time window for resetting the rate limit count.
        /// Default is 1 minute.
        /// </summary>
        [JsonProperty(PropertyName = "rateLimitDurationInMinutes")]
        public int? RateLimitDurationInMinutes { get; set; }

        /// <summary>
        /// Gets or sets number of allowed requests per client within the time
        /// window.
        /// </summary>
        [JsonProperty(PropertyName = "rateLimitThreshold")]
        public int? RateLimitThreshold { get; set; }

        /// <summary>
        /// Gets or sets list of match conditions.
        /// </summary>
        [JsonProperty(PropertyName = "matchConditions")]
        public IList<MatchCondition> MatchConditions { get; set; }

        /// <summary>
        /// Gets or sets describes what action to be applied when rule matches.
        /// Possible values include: 'Allow', 'Block', 'Log', 'Redirect'
        /// </summary>
        [JsonProperty(PropertyName = "action")]
        public string Action { get; set; }

        /// <summary>
        /// Validate the object.
        /// </summary>
        /// <exception cref="ValidationException">
        /// Thrown if validation fails
        /// </exception>
        public virtual void Validate()
        {
            if (RuleType == null)
            {
                throw new ValidationException(ValidationRules.CannotBeNull, "RuleType");
            }
            if (MatchConditions == null)
            {
                throw new ValidationException(ValidationRules.CannotBeNull, "MatchConditions");
            }
            if (Action == null)
            {
                throw new ValidationException(ValidationRules.CannotBeNull, "Action");
            }
            if (Name != null)
            {
                if (Name.Length > 128)
                {
                    throw new ValidationException(ValidationRules.MaxLength, "Name", 128);
                }
            }
            if (RateLimitDurationInMinutes > 5)
            {
                throw new ValidationException(ValidationRules.InclusiveMaximum, "RateLimitDurationInMinutes", 5);
            }
            if (RateLimitDurationInMinutes < 0)
            {
                throw new ValidationException(ValidationRules.InclusiveMinimum, "RateLimitDurationInMinutes", 0);
            }
            if (RateLimitThreshold < 0)
            {
                throw new ValidationException(ValidationRules.InclusiveMinimum, "RateLimitThreshold", 0);
            }
            if (MatchConditions != null)
            {
                foreach (var element in MatchConditions)
                {
                    if (element != null)
                    {
                        element.Validate();
                    }
                }
            }
        }
    }
}
