// <auto-generated>
// Copyright (c) Microsoft Corporation. All rights reserved.
// Licensed under the MIT License. See License.txt in the project root for
// license information.
//
// Code generated by Microsoft (R) AutoRest Code Generator.
// Changes may cause incorrect behavior and will be lost if the code is
// regenerated.
// </auto-generated>

namespace Microsoft.Azure.Search.Models
{
    using Newtonsoft.Json;
    using System.Collections;
    using System.Collections.Generic;
    using System.Linq;

    /// <summary>
    /// Parameters for filtering, sorting, faceting, paging, and other search
    /// query behaviors.
    /// </summary>
    internal partial class SearchRequest
    {
        /// <summary>
        /// Initializes a new instance of the SearchRequest class.
        /// </summary>
        public SearchRequest()
        {
            CustomInit();
        }

        /// <summary>
        /// Initializes a new instance of the SearchRequest class.
        /// </summary>
        /// <param name="includeTotalResultCount">A value that specifies
        /// whether to fetch the total count of results. Default is false.
        /// Setting this value to true may have a performance impact. Note that
        /// the count returned is an approximation.</param>
        /// <param name="facets">The list of facet expressions to apply to the
        /// search query. Each facet expression contains a field name,
        /// optionally followed by a comma-separated list of name:value
        /// pairs.</param>
        /// <param name="filter">The OData $filter expression to apply to the
        /// search query.</param>
        /// <param name="highlightFields">The comma-separated list of field
        /// names to use for hit highlights. Only searchable fields can be used
        /// for hit highlighting.</param>
        /// <param name="highlightPostTag">A string tag that is appended to hit
        /// highlights. Must be set with highlightPreTag. Default is
        /// &amp;lt;/em&amp;gt;.</param>
        /// <param name="highlightPreTag">A string tag that is prepended to hit
        /// highlights. Must be set with highlightPostTag. Default is
        /// &amp;lt;em&amp;gt;.</param>
        /// <param name="minimumCoverage">A number between 0 and 100 indicating
        /// the percentage of the index that must be covered by a search query
        /// in order for the query to be reported as a success. This parameter
        /// can be useful for ensuring search availability even for services
        /// with only one replica. The default is 100.</param>
        /// <param name="orderBy">The comma-separated list of OData $orderby
        /// expressions by which to sort the results. Each expression can be
        /// either a field name or a call to either the geo.distance() or the
        /// search.score() functions. Each expression can be followed by asc to
        /// indicate ascending, or desc to indicate descending. The default is
        /// ascending order. Ties will be broken by the match scores of
        /// documents. If no $orderby is specified, the default sort order is
        /// descending by document match score. There can be at most 32
        /// $orderby clauses.</param>
        /// <param name="queryType">A value that specifies the syntax of the
        /// search query. The default is 'simple'. Use 'full' if your query
        /// uses the Lucene query syntax. Possible values include: 'simple',
        /// 'full'</param>
        /// <param name="scoringParameters">The list of parameter values to be
        /// used in scoring functions (for example, referencePointParameter)
        /// using the format name-values. For example, if the scoring profile
        /// defines a function with a parameter called 'mylocation' the
        /// parameter string would be "mylocation--122.2,44.8" (without the
        /// quotes).</param>
        /// <param name="scoringProfile">The name of a scoring profile to
        /// evaluate match scores for matching documents in order to sort the
        /// results.</param>
        /// <param name="searchText">A full-text search query expression; Use
        /// "*" or omit this parameter to match all documents.</param>
        /// <param name="searchFields">The comma-separated list of field names
        /// to which to scope the full-text search. When using fielded search
        /// (fieldName:searchExpression) in a full Lucene query, the field
        /// names of each fielded search expression take precedence over any
        /// field names listed in this parameter.</param>
        /// <param name="searchMode">A value that specifies whether any or all
        /// of the search terms must be matched in order to count the document
        /// as a match. Possible values include: 'any', 'all'</param>
        /// <param name="select">The comma-separated list of fields to
        /// retrieve. If unspecified, all fields marked as retrievable in the
        /// schema are included.</param>
        /// <param name="skip">The number of search results to skip. This value
        /// cannot be greater than 100,000. If you need to scan documents in
        /// sequence, but cannot use skip due to this limitation, consider
        /// using orderby on a totally-ordered key and filter with a range
        /// query instead.</param>
        /// <param name="top">The number of search results to retrieve. This
        /// can be used in conjunction with $skip to implement client-side
        /// paging of search results. If results are truncated due to
        /// server-side paging, the response will include a continuation token
        /// that can be used to issue another Search request for the next page
        /// of results.</param>
        public SearchRequest(bool? includeTotalResultCount = default(bool?), IList<string> facets = default(IList<string>), string filter = default(string), string highlightFields = default(string), string highlightPostTag = default(string), string highlightPreTag = default(string), double? minimumCoverage = default(double?), string orderBy = default(string), QueryType? queryType = default(QueryType?), IList<string> scoringParameters = default(IList<string>), string scoringProfile = default(string), string searchText = default(string), string searchFields = default(string), SearchMode? searchMode = default(SearchMode?), string select = default(string), int? skip = default(int?), int? top = default(int?))
        {
            IncludeTotalResultCount = includeTotalResultCount;
            Facets = facets;
            Filter = filter;
            HighlightFields = highlightFields;
            HighlightPostTag = highlightPostTag;
            HighlightPreTag = highlightPreTag;
            MinimumCoverage = minimumCoverage;
            OrderBy = orderBy;
            QueryType = queryType;
            ScoringParameters = scoringParameters;
            ScoringProfile = scoringProfile;
            SearchText = searchText;
            SearchFields = searchFields;
            SearchMode = searchMode;
            Select = select;
            Skip = skip;
            Top = top;
            CustomInit();
        }

        /// <summary>
        /// An initialization method that performs custom operations like setting defaults
        /// </summary>
        partial void CustomInit();

        /// <summary>
        /// Gets or sets a value that specifies whether to fetch the total
        /// count of results. Default is false. Setting this value to true may
        /// have a performance impact. Note that the count returned is an
        /// approximation.
        /// </summary>
        [JsonProperty(PropertyName = "count")]
        public bool? IncludeTotalResultCount { get; set; }

        /// <summary>
        /// Gets or sets the list of facet expressions to apply to the search
        /// query. Each facet expression contains a field name, optionally
        /// followed by a comma-separated list of name:value pairs.
        /// </summary>
        [JsonProperty(PropertyName = "facets")]
        public IList<string> Facets { get; set; }

        /// <summary>
        /// Gets or sets the OData $filter expression to apply to the search
        /// query.
        /// </summary>
        [JsonProperty(PropertyName = "filter")]
        public string Filter { get; set; }

        /// <summary>
        /// Gets or sets the comma-separated list of field names to use for hit
        /// highlights. Only searchable fields can be used for hit
        /// highlighting.
        /// </summary>
        [JsonProperty(PropertyName = "highlight")]
        public string HighlightFields { get; set; }

        /// <summary>
        /// Gets or sets a string tag that is appended to hit highlights. Must
        /// be set with highlightPreTag. Default is
        /// &amp;amp;lt;/em&amp;amp;gt;.
        /// </summary>
        [JsonProperty(PropertyName = "highlightPostTag")]
        public string HighlightPostTag { get; set; }

        /// <summary>
        /// Gets or sets a string tag that is prepended to hit highlights. Must
        /// be set with highlightPostTag. Default is
        /// &amp;amp;lt;em&amp;amp;gt;.
        /// </summary>
        [JsonProperty(PropertyName = "highlightPreTag")]
        public string HighlightPreTag { get; set; }

        /// <summary>
        /// Gets or sets a number between 0 and 100 indicating the percentage
        /// of the index that must be covered by a search query in order for
        /// the query to be reported as a success. This parameter can be useful
        /// for ensuring search availability even for services with only one
        /// replica. The default is 100.
        /// </summary>
        [JsonProperty(PropertyName = "minimumCoverage")]
        public double? MinimumCoverage { get; set; }

        /// <summary>
        /// Gets or sets the comma-separated list of OData $orderby expressions
        /// by which to sort the results. Each expression can be either a field
        /// name or a call to either the geo.distance() or the search.score()
        /// functions. Each expression can be followed by asc to indicate
        /// ascending, or desc to indicate descending. The default is ascending
        /// order. Ties will be broken by the match scores of documents. If no
        /// $orderby is specified, the default sort order is descending by
        /// document match score. There can be at most 32 $orderby clauses.
        /// </summary>
        [JsonProperty(PropertyName = "orderby")]
        public string OrderBy { get; set; }

        /// <summary>
        /// Gets or sets a value that specifies the syntax of the search query.
        /// The default is 'simple'. Use 'full' if your query uses the Lucene
        /// query syntax. Possible values include: 'simple', 'full'
        /// </summary>
        [JsonProperty(PropertyName = "queryType")]
        public QueryType? QueryType { get; set; }

        /// <summary>
        /// Gets or sets the list of parameter values to be used in scoring
        /// functions (for example, referencePointParameter) using the format
        /// name-values. For example, if the scoring profile defines a function
        /// with a parameter called 'mylocation' the parameter string would be
        /// "mylocation--122.2,44.8" (without the quotes).
        /// </summary>
        [JsonProperty(PropertyName = "scoringParameters")]
        public IList<string> ScoringParameters { get; set; }

        /// <summary>
        /// Gets or sets the name of a scoring profile to evaluate match scores
        /// for matching documents in order to sort the results.
        /// </summary>
        [JsonProperty(PropertyName = "scoringProfile")]
        public string ScoringProfile { get; set; }

        /// <summary>
        /// Gets or sets a full-text search query expression; Use "*" or omit
        /// this parameter to match all documents.
        /// </summary>
        [JsonProperty(PropertyName = "search")]
        public string SearchText { get; set; }

        /// <summary>
        /// Gets or sets the comma-separated list of field names to which to
        /// scope the full-text search. When using fielded search
        /// (fieldName:searchExpression) in a full Lucene query, the field
        /// names of each fielded search expression take precedence over any
        /// field names listed in this parameter.
        /// </summary>
        [JsonProperty(PropertyName = "searchFields")]
        public string SearchFields { get; set; }

        /// <summary>
        /// Gets or sets a value that specifies whether any or all of the
        /// search terms must be matched in order to count the document as a
        /// match. Possible values include: 'any', 'all'
        /// </summary>
        [JsonProperty(PropertyName = "searchMode")]
        public SearchMode? SearchMode { get; set; }

        /// <summary>
        /// Gets or sets the comma-separated list of fields to retrieve. If
        /// unspecified, all fields marked as retrievable in the schema are
        /// included.
        /// </summary>
        [JsonProperty(PropertyName = "select")]
        public string Select { get; set; }

        /// <summary>
        /// Gets or sets the number of search results to skip. This value
        /// cannot be greater than 100,000. If you need to scan documents in
        /// sequence, but cannot use skip due to this limitation, consider
        /// using orderby on a totally-ordered key and filter with a range
        /// query instead.
        /// </summary>
        [JsonProperty(PropertyName = "skip")]
        public int? Skip { get; set; }

        /// <summary>
        /// Gets or sets the number of search results to retrieve. This can be
        /// used in conjunction with $skip to implement client-side paging of
        /// search results. If results are truncated due to server-side paging,
        /// the response will include a continuation token that can be used to
        /// issue another Search request for the next page of results.
        /// </summary>
        [JsonProperty(PropertyName = "top")]
        public int? Top { get; set; }

    }
}
