﻿// Copyright (c) Microsoft Corporation. All rights reserved.
// Licensed under the MIT License. See License.txt in the project root for
// license information.

using System;
using Microsoft.Azure.Search.Models;

namespace Microsoft.Azure.Search
{
    /// <summary>
    /// Indicates that the <see cref="Field"/> generated by <see cref="FieldBuilder"/> for
    /// the target property should have its <see cref="Field.Analyzer"/> property set to the
    /// specified analyzer.
    /// </summary>
    [AttributeUsage(AttributeTargets.Property)]
    public class AnalyzerAttribute : Attribute
    {
        /// <summary>
        /// Indicates that the specified analyzer should be used.
        /// </summary>
        /// <param name="analyzerName">
        /// The name of the analyzer. Use one of the names in <c cref="AnalyzerName.AsString">AnalyzerName.AsString</c>
        /// or the name of a custom analyzer.
        /// </param>
        public AnalyzerAttribute(string analyzerName)
        {
            Name = analyzerName;
        }

        /// <summary>
        /// The name of the analyzer.
        /// </summary>
        public string Name { get; }
    }
}
