// Copyright (c) Microsoft Corporation. All rights reserved.
// Licensed under the MIT License. See License.txt in the project root for
// license information.
//
// Code generated by Microsoft (R) AutoRest Code Generator.
// Changes may cause incorrect behavior and will be lost if the code is
// regenerated.

namespace Microsoft.Azure.Management.StreamAnalytics
{
    using Microsoft.Azure;
    using Microsoft.Azure.Management;
    using Microsoft.Rest;
    using Microsoft.Rest.Azure;
    using Models;
    using System.Collections;
    using System.Collections.Generic;
    using System.Threading;
    using System.Threading.Tasks;

    /// <summary>
    /// InputsOperations operations.
    /// </summary>
    public partial interface IInputsOperations
    {
        /// <summary>
        /// Creates an input or replaces an already existing input under an
        /// existing streaming job.
        /// </summary>
        /// <param name='input'>
        /// The definition of the input that will be used to create a new input
        /// or replace the existing one under the streaming job.
        /// </param>
        /// <param name='resourceGroupName'>
        /// The name of the resource group that contains the resource. You can
        /// obtain this value from the Azure Resource Manager API or the
        /// portal.
        /// </param>
        /// <param name='jobName'>
        /// The name of the streaming job.
        /// </param>
        /// <param name='inputName'>
        /// The name of the input.
        /// </param>
        /// <param name='ifMatch'>
        /// The ETag of the input. Omit this value to always overwrite the
        /// current input. Specify the last-seen ETag value to prevent
        /// accidentally overwritting concurrent changes.
        /// </param>
        /// <param name='ifNoneMatch'>
        /// Set to '*' to allow a new input to be created, but to prevent
        /// updating an existing input. Other values will result in a 412
        /// Pre-condition Failed response.
        /// </param>
        /// <param name='customHeaders'>
        /// The headers that will be added to request.
        /// </param>
        /// <param name='cancellationToken'>
        /// The cancellation token.
        /// </param>
        /// <exception cref="Microsoft.Rest.Azure.CloudException">
        /// Thrown when the operation returned an invalid status code
        /// </exception>
        /// <exception cref="Microsoft.Rest.SerializationException">
        /// Thrown when unable to deserialize the response
        /// </exception>
        /// <exception cref="Microsoft.Rest.ValidationException">
        /// Thrown when a required parameter is null
        /// </exception>
        Task<AzureOperationResponse<Input,InputsCreateOrReplaceHeaders>> CreateOrReplaceWithHttpMessagesAsync(Input input, string resourceGroupName, string jobName, string inputName, string ifMatch = default(string), string ifNoneMatch = default(string), Dictionary<string, List<string>> customHeaders = null, CancellationToken cancellationToken = default(CancellationToken));
        /// <summary>
        /// Updates an existing input under an existing streaming job. This can
        /// be used to partially update (ie. update one or two properties) an
        /// input without affecting the rest the job or input definition.
        /// </summary>
        /// <param name='input'>
        /// An Input object. The properties specified here will overwrite the
        /// corresponding properties in the existing input (ie. Those
        /// properties will be updated). Any properties that are set to null
        /// here will mean that the corresponding property in the existing
        /// input will remain the same and not change as a result of this PATCH
        /// operation.
        /// </param>
        /// <param name='resourceGroupName'>
        /// The name of the resource group that contains the resource. You can
        /// obtain this value from the Azure Resource Manager API or the
        /// portal.
        /// </param>
        /// <param name='jobName'>
        /// The name of the streaming job.
        /// </param>
        /// <param name='inputName'>
        /// The name of the input.
        /// </param>
        /// <param name='ifMatch'>
        /// The ETag of the input. Omit this value to always overwrite the
        /// current input. Specify the last-seen ETag value to prevent
        /// accidentally overwritting concurrent changes.
        /// </param>
        /// <param name='customHeaders'>
        /// The headers that will be added to request.
        /// </param>
        /// <param name='cancellationToken'>
        /// The cancellation token.
        /// </param>
        /// <exception cref="Microsoft.Rest.Azure.CloudException">
        /// Thrown when the operation returned an invalid status code
        /// </exception>
        /// <exception cref="Microsoft.Rest.SerializationException">
        /// Thrown when unable to deserialize the response
        /// </exception>
        /// <exception cref="Microsoft.Rest.ValidationException">
        /// Thrown when a required parameter is null
        /// </exception>
        Task<AzureOperationResponse<Input,InputsUpdateHeaders>> UpdateWithHttpMessagesAsync(Input input, string resourceGroupName, string jobName, string inputName, string ifMatch = default(string), Dictionary<string, List<string>> customHeaders = null, CancellationToken cancellationToken = default(CancellationToken));
        /// <summary>
        /// Deletes an input from the streaming job.
        /// </summary>
        /// <param name='resourceGroupName'>
        /// The name of the resource group that contains the resource. You can
        /// obtain this value from the Azure Resource Manager API or the
        /// portal.
        /// </param>
        /// <param name='jobName'>
        /// The name of the streaming job.
        /// </param>
        /// <param name='inputName'>
        /// The name of the input.
        /// </param>
        /// <param name='customHeaders'>
        /// The headers that will be added to request.
        /// </param>
        /// <param name='cancellationToken'>
        /// The cancellation token.
        /// </param>
        /// <exception cref="Microsoft.Rest.Azure.CloudException">
        /// Thrown when the operation returned an invalid status code
        /// </exception>
        /// <exception cref="Microsoft.Rest.ValidationException">
        /// Thrown when a required parameter is null
        /// </exception>
        Task<AzureOperationResponse> DeleteWithHttpMessagesAsync(string resourceGroupName, string jobName, string inputName, Dictionary<string, List<string>> customHeaders = null, CancellationToken cancellationToken = default(CancellationToken));
        /// <summary>
        /// Gets details about the specified input.
        /// </summary>
        /// <param name='resourceGroupName'>
        /// The name of the resource group that contains the resource. You can
        /// obtain this value from the Azure Resource Manager API or the
        /// portal.
        /// </param>
        /// <param name='jobName'>
        /// The name of the streaming job.
        /// </param>
        /// <param name='inputName'>
        /// The name of the input.
        /// </param>
        /// <param name='customHeaders'>
        /// The headers that will be added to request.
        /// </param>
        /// <param name='cancellationToken'>
        /// The cancellation token.
        /// </param>
        /// <exception cref="Microsoft.Rest.Azure.CloudException">
        /// Thrown when the operation returned an invalid status code
        /// </exception>
        /// <exception cref="Microsoft.Rest.SerializationException">
        /// Thrown when unable to deserialize the response
        /// </exception>
        /// <exception cref="Microsoft.Rest.ValidationException">
        /// Thrown when a required parameter is null
        /// </exception>
        Task<AzureOperationResponse<Input,InputsGetHeaders>> GetWithHttpMessagesAsync(string resourceGroupName, string jobName, string inputName, Dictionary<string, List<string>> customHeaders = null, CancellationToken cancellationToken = default(CancellationToken));
        /// <summary>
        /// Lists all of the inputs under the specified streaming job.
        /// </summary>
        /// <param name='resourceGroupName'>
        /// The name of the resource group that contains the resource. You can
        /// obtain this value from the Azure Resource Manager API or the
        /// portal.
        /// </param>
        /// <param name='jobName'>
        /// The name of the streaming job.
        /// </param>
        /// <param name='select'>
        /// The $select OData query parameter. This is a comma-separated list
        /// of structural properties to include in the response, or “*” to
        /// include all properties. By default, all properties are returned
        /// except diagnostics. Currently only accepts '*' as a valid value.
        /// </param>
        /// <param name='customHeaders'>
        /// The headers that will be added to request.
        /// </param>
        /// <param name='cancellationToken'>
        /// The cancellation token.
        /// </param>
        /// <exception cref="Microsoft.Rest.Azure.CloudException">
        /// Thrown when the operation returned an invalid status code
        /// </exception>
        /// <exception cref="Microsoft.Rest.SerializationException">
        /// Thrown when unable to deserialize the response
        /// </exception>
        /// <exception cref="Microsoft.Rest.ValidationException">
        /// Thrown when a required parameter is null
        /// </exception>
        Task<AzureOperationResponse<IPage<Input>>> ListByStreamingJobWithHttpMessagesAsync(string resourceGroupName, string jobName, string select = default(string), Dictionary<string, List<string>> customHeaders = null, CancellationToken cancellationToken = default(CancellationToken));
        /// <summary>
        /// Tests whether an input’s datasource is reachable and usable by the
        /// Azure Stream Analytics service.
        /// </summary>
        /// <param name='resourceGroupName'>
        /// The name of the resource group that contains the resource. You can
        /// obtain this value from the Azure Resource Manager API or the
        /// portal.
        /// </param>
        /// <param name='jobName'>
        /// The name of the streaming job.
        /// </param>
        /// <param name='inputName'>
        /// The name of the input.
        /// </param>
        /// <param name='input'>
        /// If the input specified does not already exist, this parameter must
        /// contain the full input definition intended to be tested. If the
        /// input specified already exists, this parameter can be left null to
        /// test the existing input as is or if specified, the properties
        /// specified will overwrite the corresponding properties in the
        /// existing input (exactly like a PATCH operation) and the resulting
        /// input will be tested.
        /// </param>
        /// <param name='customHeaders'>
        /// The headers that will be added to request.
        /// </param>
        /// <param name='cancellationToken'>
        /// The cancellation token.
        /// </param>
        /// <exception cref="Microsoft.Rest.Azure.CloudException">
        /// Thrown when the operation returned an invalid status code
        /// </exception>
        /// <exception cref="Microsoft.Rest.SerializationException">
        /// Thrown when unable to deserialize the response
        /// </exception>
        /// <exception cref="Microsoft.Rest.ValidationException">
        /// Thrown when a required parameter is null
        /// </exception>
        Task<AzureOperationResponse<ResourceTestStatus>> TestWithHttpMessagesAsync(string resourceGroupName, string jobName, string inputName, Input input = default(Input), Dictionary<string, List<string>> customHeaders = null, CancellationToken cancellationToken = default(CancellationToken));
        /// <summary>
        /// Tests whether an input’s datasource is reachable and usable by the
        /// Azure Stream Analytics service.
        /// </summary>
        /// <param name='resourceGroupName'>
        /// The name of the resource group that contains the resource. You can
        /// obtain this value from the Azure Resource Manager API or the
        /// portal.
        /// </param>
        /// <param name='jobName'>
        /// The name of the streaming job.
        /// </param>
        /// <param name='inputName'>
        /// The name of the input.
        /// </param>
        /// <param name='input'>
        /// If the input specified does not already exist, this parameter must
        /// contain the full input definition intended to be tested. If the
        /// input specified already exists, this parameter can be left null to
        /// test the existing input as is or if specified, the properties
        /// specified will overwrite the corresponding properties in the
        /// existing input (exactly like a PATCH operation) and the resulting
        /// input will be tested.
        /// </param>
        /// <param name='customHeaders'>
        /// The headers that will be added to request.
        /// </param>
        /// <param name='cancellationToken'>
        /// The cancellation token.
        /// </param>
        /// <exception cref="Microsoft.Rest.Azure.CloudException">
        /// Thrown when the operation returned an invalid status code
        /// </exception>
        /// <exception cref="Microsoft.Rest.SerializationException">
        /// Thrown when unable to deserialize the response
        /// </exception>
        /// <exception cref="Microsoft.Rest.ValidationException">
        /// Thrown when a required parameter is null
        /// </exception>
        Task<AzureOperationResponse<ResourceTestStatus>> BeginTestWithHttpMessagesAsync(string resourceGroupName, string jobName, string inputName, Input input = default(Input), Dictionary<string, List<string>> customHeaders = null, CancellationToken cancellationToken = default(CancellationToken));
        /// <summary>
        /// Lists all of the inputs under the specified streaming job.
        /// </summary>
        /// <param name='nextPageLink'>
        /// The NextLink from the previous successful call to List operation.
        /// </param>
        /// <param name='customHeaders'>
        /// The headers that will be added to request.
        /// </param>
        /// <param name='cancellationToken'>
        /// The cancellation token.
        /// </param>
        /// <exception cref="Microsoft.Rest.Azure.CloudException">
        /// Thrown when the operation returned an invalid status code
        /// </exception>
        /// <exception cref="Microsoft.Rest.SerializationException">
        /// Thrown when unable to deserialize the response
        /// </exception>
        /// <exception cref="Microsoft.Rest.ValidationException">
        /// Thrown when a required parameter is null
        /// </exception>
        Task<AzureOperationResponse<IPage<Input>>> ListByStreamingJobNextWithHttpMessagesAsync(string nextPageLink, Dictionary<string, List<string>> customHeaders = null, CancellationToken cancellationToken = default(CancellationToken));
    }
}
