// Copyright (c) Microsoft Corporation. All rights reserved.
// Licensed under the MIT License. See License.txt in the project root for
// license information.
//
// Code generated by Microsoft (R) AutoRest Code Generator.
// Changes may cause incorrect behavior and will be lost if the code is
// regenerated.

namespace Microsoft.Azure.Management.StreamAnalytics.Models
{
    using Microsoft.Azure;
    using Microsoft.Azure.Management;
    using Microsoft.Azure.Management.StreamAnalytics;
    using Microsoft.Rest;
    using Microsoft.Rest.Serialization;
    using Newtonsoft.Json;
    using System.Collections;
    using System.Collections.Generic;
    using System.Linq;

    /// <summary>
    /// Describes an Azure Table output data source.
    /// </summary>
    [Newtonsoft.Json.JsonObject("Microsoft.Storage/Table")]
    [Rest.Serialization.JsonTransformation]
    public partial class AzureTableOutputDataSource : OutputDataSource
    {
        /// <summary>
        /// Initializes a new instance of the AzureTableOutputDataSource class.
        /// </summary>
        public AzureTableOutputDataSource()
        {
          CustomInit();
        }

        /// <summary>
        /// Initializes a new instance of the AzureTableOutputDataSource class.
        /// </summary>
        /// <param name="accountName">The name of the Azure Storage account.
        /// Required on PUT (CreateOrReplace) requests.</param>
        /// <param name="accountKey">The account key for the Azure Storage
        /// account. Required on PUT (CreateOrReplace) requests.</param>
        /// <param name="table">The name of the Azure Table. Required on PUT
        /// (CreateOrReplace) requests.</param>
        /// <param name="partitionKey">This element indicates the name of a
        /// column from the SELECT statement in the query that will be used as
        /// the partition key for the Azure Table. Required on PUT
        /// (CreateOrReplace) requests.</param>
        /// <param name="rowKey">This element indicates the name of a column
        /// from the SELECT statement in the query that will be used as the row
        /// key for the Azure Table. Required on PUT (CreateOrReplace)
        /// requests.</param>
        /// <param name="columnsToRemove">If specified, each item in the array
        /// is the name of a column to remove (if present) from output event
        /// entities.</param>
        /// <param name="batchSize">The number of rows to write to the Azure
        /// Table at a time.</param>
        public AzureTableOutputDataSource(string accountName = default(string), string accountKey = default(string), string table = default(string), string partitionKey = default(string), string rowKey = default(string), IList<string> columnsToRemove = default(IList<string>), int? batchSize = default(int?))
        {
            AccountName = accountName;
            AccountKey = accountKey;
            Table = table;
            PartitionKey = partitionKey;
            RowKey = rowKey;
            ColumnsToRemove = columnsToRemove;
            BatchSize = batchSize;
            CustomInit();
        }

        /// <summary>
        /// An initialization method that performs custom operations like setting defaults
        /// </summary>
        partial void CustomInit();

        /// <summary>
        /// Gets or sets the name of the Azure Storage account. Required on PUT
        /// (CreateOrReplace) requests.
        /// </summary>
        [JsonProperty(PropertyName = "properties.accountName")]
        public string AccountName { get; set; }

        /// <summary>
        /// Gets or sets the account key for the Azure Storage account.
        /// Required on PUT (CreateOrReplace) requests.
        /// </summary>
        [JsonProperty(PropertyName = "properties.accountKey")]
        public string AccountKey { get; set; }

        /// <summary>
        /// Gets or sets the name of the Azure Table. Required on PUT
        /// (CreateOrReplace) requests.
        /// </summary>
        [JsonProperty(PropertyName = "properties.table")]
        public string Table { get; set; }

        /// <summary>
        /// Gets or sets this element indicates the name of a column from the
        /// SELECT statement in the query that will be used as the partition
        /// key for the Azure Table. Required on PUT (CreateOrReplace)
        /// requests.
        /// </summary>
        [JsonProperty(PropertyName = "properties.partitionKey")]
        public string PartitionKey { get; set; }

        /// <summary>
        /// Gets or sets this element indicates the name of a column from the
        /// SELECT statement in the query that will be used as the row key for
        /// the Azure Table. Required on PUT (CreateOrReplace) requests.
        /// </summary>
        [JsonProperty(PropertyName = "properties.rowKey")]
        public string RowKey { get; set; }

        /// <summary>
        /// Gets or sets if specified, each item in the array is the name of a
        /// column to remove (if present) from output event entities.
        /// </summary>
        [JsonProperty(PropertyName = "properties.columnsToRemove")]
        public IList<string> ColumnsToRemove { get; set; }

        /// <summary>
        /// Gets or sets the number of rows to write to the Azure Table at a
        /// time.
        /// </summary>
        [JsonProperty(PropertyName = "properties.batchSize")]
        public int? BatchSize { get; set; }

    }
}
