// Copyright (c) Microsoft Corporation. All rights reserved.
// Licensed under the MIT License. See License.txt in the project root for
// license information.
//
// Code generated by Microsoft (R) AutoRest Code Generator.
// Changes may cause incorrect behavior and will be lost if the code is
// regenerated.

namespace Microsoft.Azure.Management.StreamAnalytics.Models
{
    using Microsoft.Azure;
    using Microsoft.Azure.Management;
    using Microsoft.Azure.Management.StreamAnalytics;
    using Microsoft.Rest;
    using Microsoft.Rest.Serialization;
    using Newtonsoft.Json;
    using System.Linq;

    /// <summary>
    /// Describes a DocumentDB output data source.
    /// </summary>
    [Newtonsoft.Json.JsonObject("Microsoft.Storage/DocumentDB")]
    [Rest.Serialization.JsonTransformation]
    public partial class DocumentDbOutputDataSource : OutputDataSource
    {
        /// <summary>
        /// Initializes a new instance of the DocumentDbOutputDataSource class.
        /// </summary>
        public DocumentDbOutputDataSource()
        {
          CustomInit();
        }

        /// <summary>
        /// Initializes a new instance of the DocumentDbOutputDataSource class.
        /// </summary>
        /// <param name="accountId">The DocumentDB account name or ID. Required
        /// on PUT (CreateOrReplace) requests.</param>
        /// <param name="accountKey">The account key for the DocumentDB
        /// account. Required on PUT (CreateOrReplace) requests.</param>
        /// <param name="database">The name of the DocumentDB database.
        /// Required on PUT (CreateOrReplace) requests.</param>
        /// <param name="collectionNamePattern">The collection name pattern for
        /// the collections to be used. The collection name format can be
        /// constructed using the optional {partition} token, where partitions
        /// start from 0. See the DocumentDB section of
        /// https://docs.microsoft.com/en-us/rest/api/streamanalytics/stream-analytics-output
        /// for more information. Required on PUT (CreateOrReplace)
        /// requests.</param>
        /// <param name="partitionKey">The name of the field in output events
        /// used to specify the key for partitioning output across collections.
        /// If 'collectionNamePattern' contains the {partition} token, this
        /// property is required to be specified.</param>
        /// <param name="documentId">The name of the field in output events
        /// used to specify the primary key which insert or update operations
        /// are based on.</param>
        public DocumentDbOutputDataSource(string accountId = default(string), string accountKey = default(string), string database = default(string), string collectionNamePattern = default(string), string partitionKey = default(string), string documentId = default(string))
        {
            AccountId = accountId;
            AccountKey = accountKey;
            Database = database;
            CollectionNamePattern = collectionNamePattern;
            PartitionKey = partitionKey;
            DocumentId = documentId;
            CustomInit();
        }

        /// <summary>
        /// An initialization method that performs custom operations like setting defaults
        /// </summary>
        partial void CustomInit();

        /// <summary>
        /// Gets or sets the DocumentDB account name or ID. Required on PUT
        /// (CreateOrReplace) requests.
        /// </summary>
        [JsonProperty(PropertyName = "properties.accountId")]
        public string AccountId { get; set; }

        /// <summary>
        /// Gets or sets the account key for the DocumentDB account. Required
        /// on PUT (CreateOrReplace) requests.
        /// </summary>
        [JsonProperty(PropertyName = "properties.accountKey")]
        public string AccountKey { get; set; }

        /// <summary>
        /// Gets or sets the name of the DocumentDB database. Required on PUT
        /// (CreateOrReplace) requests.
        /// </summary>
        [JsonProperty(PropertyName = "properties.database")]
        public string Database { get; set; }

        /// <summary>
        /// Gets or sets the collection name pattern for the collections to be
        /// used. The collection name format can be constructed using the
        /// optional {partition} token, where partitions start from 0. See the
        /// DocumentDB section of
        /// https://docs.microsoft.com/en-us/rest/api/streamanalytics/stream-analytics-output
        /// for more information. Required on PUT (CreateOrReplace) requests.
        /// </summary>
        [JsonProperty(PropertyName = "properties.collectionNamePattern")]
        public string CollectionNamePattern { get; set; }

        /// <summary>
        /// Gets or sets the name of the field in output events used to specify
        /// the key for partitioning output across collections. If
        /// 'collectionNamePattern' contains the {partition} token, this
        /// property is required to be specified.
        /// </summary>
        [JsonProperty(PropertyName = "properties.partitionKey")]
        public string PartitionKey { get; set; }

        /// <summary>
        /// Gets or sets the name of the field in output events used to specify
        /// the primary key which insert or update operations are based on.
        /// </summary>
        [JsonProperty(PropertyName = "properties.documentId")]
        public string DocumentId { get; set; }

    }
}
