// Copyright (c) Microsoft Corporation. All rights reserved.
// Licensed under the MIT License. See License.txt in the project root for
// license information.
//
// Code generated by Microsoft (R) AutoRest Code Generator.
// Changes may cause incorrect behavior and will be lost if the code is
// regenerated.

namespace Microsoft.Azure.Management.StreamAnalytics.Models
{
    using Microsoft.Azure;
    using Microsoft.Azure.Management;
    using Microsoft.Azure.Management.StreamAnalytics;
    using Microsoft.Rest;
    using Microsoft.Rest.Serialization;
    using Newtonsoft.Json;
    using System.Linq;

    /// <summary>
    /// Describes an Event Hub input data source that contains stream data.
    /// </summary>
    [Newtonsoft.Json.JsonObject("Microsoft.ServiceBus/EventHub")]
    [Rest.Serialization.JsonTransformation]
    public partial class EventHubStreamInputDataSource : StreamInputDataSource
    {
        /// <summary>
        /// Initializes a new instance of the EventHubStreamInputDataSource
        /// class.
        /// </summary>
        public EventHubStreamInputDataSource()
        {
          CustomInit();
        }

        /// <summary>
        /// Initializes a new instance of the EventHubStreamInputDataSource
        /// class.
        /// </summary>
        /// <param name="serviceBusNamespace">The namespace that is associated
        /// with the desired Event Hub, Service Bus Queue, Service Bus Topic,
        /// etc. Required on PUT (CreateOrReplace) requests.</param>
        /// <param name="sharedAccessPolicyName">The shared access policy name
        /// for the Event Hub, Service Bus Queue, Service Bus Topic, etc.
        /// Required on PUT (CreateOrReplace) requests.</param>
        /// <param name="sharedAccessPolicyKey">The shared access policy key
        /// for the specified shared access policy. Required on PUT
        /// (CreateOrReplace) requests.</param>
        /// <param name="eventHubName">The name of the Event Hub. Required on
        /// PUT (CreateOrReplace) requests.</param>
        /// <param name="consumerGroupName">The name of an Event Hub Consumer
        /// Group that should be used to read events from the Event Hub.
        /// Specifying distinct consumer group names for multiple inputs allows
        /// each of those inputs to receive the same events from the Event Hub.
        /// If not specified, the input uses the Event Hub’s default consumer
        /// group.</param>
        public EventHubStreamInputDataSource(string serviceBusNamespace = default(string), string sharedAccessPolicyName = default(string), string sharedAccessPolicyKey = default(string), string eventHubName = default(string), string consumerGroupName = default(string))
        {
            ServiceBusNamespace = serviceBusNamespace;
            SharedAccessPolicyName = sharedAccessPolicyName;
            SharedAccessPolicyKey = sharedAccessPolicyKey;
            EventHubName = eventHubName;
            ConsumerGroupName = consumerGroupName;
            CustomInit();
        }

        /// <summary>
        /// An initialization method that performs custom operations like setting defaults
        /// </summary>
        partial void CustomInit();

        /// <summary>
        /// Gets or sets the namespace that is associated with the desired
        /// Event Hub, Service Bus Queue, Service Bus Topic, etc. Required on
        /// PUT (CreateOrReplace) requests.
        /// </summary>
        [JsonProperty(PropertyName = "properties.serviceBusNamespace")]
        public string ServiceBusNamespace { get; set; }

        /// <summary>
        /// Gets or sets the shared access policy name for the Event Hub,
        /// Service Bus Queue, Service Bus Topic, etc. Required on PUT
        /// (CreateOrReplace) requests.
        /// </summary>
        [JsonProperty(PropertyName = "properties.sharedAccessPolicyName")]
        public string SharedAccessPolicyName { get; set; }

        /// <summary>
        /// Gets or sets the shared access policy key for the specified shared
        /// access policy. Required on PUT (CreateOrReplace) requests.
        /// </summary>
        [JsonProperty(PropertyName = "properties.sharedAccessPolicyKey")]
        public string SharedAccessPolicyKey { get; set; }

        /// <summary>
        /// Gets or sets the name of the Event Hub. Required on PUT
        /// (CreateOrReplace) requests.
        /// </summary>
        [JsonProperty(PropertyName = "properties.eventHubName")]
        public string EventHubName { get; set; }

        /// <summary>
        /// Gets or sets the name of an Event Hub Consumer Group that should be
        /// used to read events from the Event Hub. Specifying distinct
        /// consumer group names for multiple inputs allows each of those
        /// inputs to receive the same events from the Event Hub. If not
        /// specified, the input uses the Event Hub’s default consumer group.
        /// </summary>
        [JsonProperty(PropertyName = "properties.consumerGroupName")]
        public string ConsumerGroupName { get; set; }

    }
}
