// Copyright (c) Microsoft Corporation. All rights reserved.
// Licensed under the MIT License. See License.txt in the project root for
// license information.
//
// Code generated by Microsoft (R) AutoRest Code Generator.
// Changes may cause incorrect behavior and will be lost if the code is
// regenerated.

namespace Microsoft.Azure.Management.StreamAnalytics.Models
{
    using Microsoft.Azure;
    using Microsoft.Azure.Management;
    using Microsoft.Azure.Management.StreamAnalytics;
    using Microsoft.Rest;
    using Microsoft.Rest.Serialization;
    using Newtonsoft.Json;
    using System.Linq;

    /// <summary>
    /// Describes an IoT Hub input data source that contains stream data.
    /// </summary>
    [Newtonsoft.Json.JsonObject("Microsoft.Devices/IotHubs")]
    [Rest.Serialization.JsonTransformation]
    public partial class IoTHubStreamInputDataSource : StreamInputDataSource
    {
        /// <summary>
        /// Initializes a new instance of the IoTHubStreamInputDataSource
        /// class.
        /// </summary>
        public IoTHubStreamInputDataSource()
        {
          CustomInit();
        }

        /// <summary>
        /// Initializes a new instance of the IoTHubStreamInputDataSource
        /// class.
        /// </summary>
        /// <param name="iotHubNamespace">The name or the URI of the IoT Hub.
        /// Required on PUT (CreateOrReplace) requests.</param>
        /// <param name="sharedAccessPolicyName">The shared access policy name
        /// for the IoT Hub. This policy must contain at least the Service
        /// connect permission. Required on PUT (CreateOrReplace)
        /// requests.</param>
        /// <param name="sharedAccessPolicyKey">The shared access policy key
        /// for the specified shared access policy. Required on PUT
        /// (CreateOrReplace) requests.</param>
        /// <param name="consumerGroupName">The name of an IoT Hub Consumer
        /// Group that should be used to read events from the IoT Hub. If not
        /// specified, the input uses the Iot Hub’s default consumer
        /// group.</param>
        /// <param name="endpoint">The IoT Hub endpoint to connect to (ie.
        /// messages/events, messages/operationsMonitoringEvents,
        /// etc.).</param>
        public IoTHubStreamInputDataSource(string iotHubNamespace = default(string), string sharedAccessPolicyName = default(string), string sharedAccessPolicyKey = default(string), string consumerGroupName = default(string), string endpoint = default(string))
        {
            IotHubNamespace = iotHubNamespace;
            SharedAccessPolicyName = sharedAccessPolicyName;
            SharedAccessPolicyKey = sharedAccessPolicyKey;
            ConsumerGroupName = consumerGroupName;
            Endpoint = endpoint;
            CustomInit();
        }

        /// <summary>
        /// An initialization method that performs custom operations like setting defaults
        /// </summary>
        partial void CustomInit();

        /// <summary>
        /// Gets or sets the name or the URI of the IoT Hub. Required on PUT
        /// (CreateOrReplace) requests.
        /// </summary>
        [JsonProperty(PropertyName = "properties.iotHubNamespace")]
        public string IotHubNamespace { get; set; }

        /// <summary>
        /// Gets or sets the shared access policy name for the IoT Hub. This
        /// policy must contain at least the Service connect permission.
        /// Required on PUT (CreateOrReplace) requests.
        /// </summary>
        [JsonProperty(PropertyName = "properties.sharedAccessPolicyName")]
        public string SharedAccessPolicyName { get; set; }

        /// <summary>
        /// Gets or sets the shared access policy key for the specified shared
        /// access policy. Required on PUT (CreateOrReplace) requests.
        /// </summary>
        [JsonProperty(PropertyName = "properties.sharedAccessPolicyKey")]
        public string SharedAccessPolicyKey { get; set; }

        /// <summary>
        /// Gets or sets the name of an IoT Hub Consumer Group that should be
        /// used to read events from the IoT Hub. If not specified, the input
        /// uses the Iot Hub’s default consumer group.
        /// </summary>
        [JsonProperty(PropertyName = "properties.consumerGroupName")]
        public string ConsumerGroupName { get; set; }

        /// <summary>
        /// Gets or sets the IoT Hub endpoint to connect to (ie.
        /// messages/events, messages/operationsMonitoringEvents, etc.).
        /// </summary>
        [JsonProperty(PropertyName = "properties.endpoint")]
        public string Endpoint { get; set; }

    }
}
