// Copyright (c) Microsoft Corporation. All rights reserved.
// Licensed under the MIT License. See License.txt in the project root for
// license information.
//
// Code generated by Microsoft (R) AutoRest Code Generator.
// Changes may cause incorrect behavior and will be lost if the code is
// regenerated.

namespace Microsoft.Azure.Management.StreamAnalytics.Models
{
    using Microsoft.Azure;
    using Microsoft.Azure.Management;
    using Microsoft.Azure.Management.StreamAnalytics;
    using Microsoft.Rest;
    using Microsoft.Rest.Serialization;
    using Newtonsoft.Json;
    using System.Linq;

    /// <summary>
    /// Describes a Power BI output data source.
    /// </summary>
    [Newtonsoft.Json.JsonObject("PowerBI")]
    [Rest.Serialization.JsonTransformation]
    public partial class PowerBIOutputDataSource : OutputDataSource
    {
        /// <summary>
        /// Initializes a new instance of the PowerBIOutputDataSource class.
        /// </summary>
        public PowerBIOutputDataSource()
        {
          CustomInit();
        }

        /// <summary>
        /// Initializes a new instance of the PowerBIOutputDataSource class.
        /// </summary>
        /// <param name="refreshToken">A refresh token that can be used to
        /// obtain a valid access token that can then be used to authenticate
        /// with the data source. A valid refresh token is currently only
        /// obtainable via the Azure Portal. It is recommended to put a dummy
        /// string value here when creating the data source and then going to
        /// the Azure Portal to authenticate the data source which will update
        /// this property with a valid refresh token. Required on PUT
        /// (CreateOrReplace) requests.</param>
        /// <param name="tokenUserPrincipalName">The user principal name (UPN)
        /// of the user that was used to obtain the refresh token. Use this
        /// property to help remember which user was used to obtain the refresh
        /// token.</param>
        /// <param name="tokenUserDisplayName">The user display name of the
        /// user that was used to obtain the refresh token. Use this property
        /// to help remember which user was used to obtain the refresh
        /// token.</param>
        /// <param name="dataset">The name of the Power BI dataset. Required on
        /// PUT (CreateOrReplace) requests.</param>
        /// <param name="table">The name of the Power BI table under the
        /// specified dataset. Required on PUT (CreateOrReplace)
        /// requests.</param>
        /// <param name="groupId">The ID of the Power BI group.</param>
        /// <param name="groupName">The name of the Power BI group. Use this
        /// property to help remember which specific Power BI group id was
        /// used.</param>
        public PowerBIOutputDataSource(string refreshToken = default(string), string tokenUserPrincipalName = default(string), string tokenUserDisplayName = default(string), string dataset = default(string), string table = default(string), string groupId = default(string), string groupName = default(string))
        {
            RefreshToken = refreshToken;
            TokenUserPrincipalName = tokenUserPrincipalName;
            TokenUserDisplayName = tokenUserDisplayName;
            Dataset = dataset;
            Table = table;
            GroupId = groupId;
            GroupName = groupName;
            CustomInit();
        }

        /// <summary>
        /// An initialization method that performs custom operations like setting defaults
        /// </summary>
        partial void CustomInit();

        /// <summary>
        /// Gets or sets a refresh token that can be used to obtain a valid
        /// access token that can then be used to authenticate with the data
        /// source. A valid refresh token is currently only obtainable via the
        /// Azure Portal. It is recommended to put a dummy string value here
        /// when creating the data source and then going to the Azure Portal to
        /// authenticate the data source which will update this property with a
        /// valid refresh token. Required on PUT (CreateOrReplace) requests.
        /// </summary>
        [JsonProperty(PropertyName = "properties.refreshToken")]
        public string RefreshToken { get; set; }

        /// <summary>
        /// Gets or sets the user principal name (UPN) of the user that was
        /// used to obtain the refresh token. Use this property to help
        /// remember which user was used to obtain the refresh token.
        /// </summary>
        [JsonProperty(PropertyName = "properties.tokenUserPrincipalName")]
        public string TokenUserPrincipalName { get; set; }

        /// <summary>
        /// Gets or sets the user display name of the user that was used to
        /// obtain the refresh token. Use this property to help remember which
        /// user was used to obtain the refresh token.
        /// </summary>
        [JsonProperty(PropertyName = "properties.tokenUserDisplayName")]
        public string TokenUserDisplayName { get; set; }

        /// <summary>
        /// Gets or sets the name of the Power BI dataset. Required on PUT
        /// (CreateOrReplace) requests.
        /// </summary>
        [JsonProperty(PropertyName = "properties.dataset")]
        public string Dataset { get; set; }

        /// <summary>
        /// Gets or sets the name of the Power BI table under the specified
        /// dataset. Required on PUT (CreateOrReplace) requests.
        /// </summary>
        [JsonProperty(PropertyName = "properties.table")]
        public string Table { get; set; }

        /// <summary>
        /// Gets or sets the ID of the Power BI group.
        /// </summary>
        [JsonProperty(PropertyName = "properties.groupId")]
        public string GroupId { get; set; }

        /// <summary>
        /// Gets or sets the name of the Power BI group. Use this property to
        /// help remember which specific Power BI group id was used.
        /// </summary>
        [JsonProperty(PropertyName = "properties.groupName")]
        public string GroupName { get; set; }

    }
}
